from udsoncan.client import Client
from udsoncan import services, CommunicationType
from udsoncan.exceptions import *

from test.ClientServerTest import ClientServerTest

class TestCommunicationControl(ClientServerTest):
	def __init__(self, *args, **kwargs):
		ClientServerTest.__init__(self, *args, **kwargs)

	def test_comcontrol_enable_node(self):
		for i in range(3):
				request = self.conn.touserqueue.get(timeout=0.2)
				self.assertEqual(request, b"\x28\x00\x01")
				self.conn.fromuserqueue.put(b"\x68\x00")	# Positive response

	def _test_comcontrol_enable_node(self):
		control_type = services.CommunicationControl.ControlType.enableRxAndTx
		com_type = CommunicationType(subnet=CommunicationType.Subnet.node, normal_msg=True)
		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)
		self.assertTrue(response.positive)
		self.assertEqual(response.service_data.control_type_echo, control_type)

		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type.get_byte())
		self.assertTrue(response.positive)
		self.assertEqual(response.service_data.control_type_echo, control_type)

		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type.get_byte_as_int())
		self.assertTrue(response.positive)
		self.assertEqual(response.service_data.control_type_echo, control_type)



	def test_comcontrol_enable_node_spr(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x28\x80\x01")
		self.conn.fromuserqueue.put('wait')	# Synchronize

	def _test_comcontrol_enable_node_spr(self):
		control_type = services.CommunicationControl.ControlType.enableRxAndTx
		com_type = CommunicationType(subnet=CommunicationType.Subnet.node, normal_msg=True)
		with self.udsclient.suppress_positive_response:
			response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)
			self.assertEqual(response, None)
		self.conn.fromuserqueue.get(timeout=0.2)	#Avoid closing connection prematurely

	def test_comcontrol_disable_subnet(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x28\x03\x33")
		self.conn.fromuserqueue.put(b"\x68\x03")	# Positive response

	def _test_comcontrol_disable_subnet(self):
		control_type = services.CommunicationControl.ControlType.disableRxAndTx
		com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	
		self.assertTrue(response.positive)
		self.assertEqual(response.service_data.control_type_echo, control_type)

	def test_comcontrol_negative_response_exception(self):
		self.wait_request_and_respond(b"\x7F\x28\x31") 	# Request Out Of Range

	def _test_comcontrol_negative_response_exception(self):
		with self.assertRaises(NegativeResponseException) as handle:
			control_type = services.CommunicationControl.ControlType.disableRxAndTx
			com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
			self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	

	def test_comcontrol_negative_response_no_exception(self):
		self.wait_request_and_respond(b"\x7F\x28\x31") 	# Request Out Of Range

	def _test_comcontrol_negative_response_no_exception(self):
		self.udsclient.config['exception_on_negative_response'] = False
		control_type = services.CommunicationControl.ControlType.disableRxAndTx
		com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)				
		self.assertTrue(response.valid)
		self.assertFalse(response.positive)
		
	def test_set_params_invalidservice_exception(self):
		self.wait_request_and_respond(b"\x00\x22") #Inexistent Service

	def _test_set_params_invalidservice_exception(self):
		with self.assertRaises(InvalidResponseException) as handle:
			control_type = services.CommunicationControl.ControlType.disableRxAndTx
			com_type = CommunicationType(subnet=5, normal_msg=True, network_management_msg=True)
			self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	
		
	def test_set_params_invalidservice_no_exception(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.conn.fromuserqueue.put(b"\x00\x22") #Inexistent Service

	def _test_set_params_invalidservice_no_exception(self):
		self.udsclient.config['exception_on_invalid_response'] = False
		control_type = services.CommunicationControl.ControlType.disableRxAndTx
		com_type = CommunicationType(subnet=5, normal_msg=True, network_management_msg=True)
		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	
		self.assertFalse(response.valid)

	def test_comcontrol_wrongservice_exception(self):
		self.wait_request_and_respond(b"\x7E\x22") # Valid but wrong service (Tester Present)

	def _test_comcontrol_wrongservice_exception(self):
		with self.assertRaises(UnexpectedResponseException) as handle:
			control_type = services.CommunicationControl.ControlType.disableRxAndTx
			com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
			self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	

	def test_comcontrol_wrongservice_no_exception(self):
		self.wait_request_and_respond(b"\x7E\x22") # Valid but wrong service (Tester Present)

	def _test_comcontrol_wrongservice_no_exception(self):
		self.udsclient.config['exception_on_unexpected_response'] = False
		control_type = services.CommunicationControl.ControlType.disableRxAndTx
		com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		response = self.udsclient.communication_control(control_type=control_type, communication_type=com_type)	
		self.assertTrue(response.valid)
		self.assertTrue(response.unexpected)

	def test_comcontrol_bad_control_type_exception(self):
		self.wait_request_and_respond(b"\x68\x08") # Valid but bad control type

	def _test_comcontrol_bad_control_type_exception(self):
		with self.assertRaises(UnexpectedResponseException) as handle:
			com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
			self.udsclient.communication_control(control_type=9, communication_type=com_type)	

	def test_comcontrol_bad_control_type_no_exception(self):
		self.wait_request_and_respond(b"\x68\x08") # Valid but bad control type

	def _test_comcontrol_bad_control_type_no_exception(self):
		self.udsclient.config['exception_on_unexpected_response'] = False
		com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		response = self.udsclient.communication_control(control_type=9, communication_type=com_type)	
		self.assertTrue(response.valid)
		self.assertTrue(response.unexpected)

	def test_bad_param(self):
		pass

	def _test_bad_param(self):
		valid_com_type = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		with self.assertRaises(ValueError):
			self.udsclient.communication_control(control_type='x', communication_type=valid_com_type)	

		with self.assertRaises(ValueError):
			self.udsclient.communication_control(control_type=0x80, communication_type=valid_com_type)

		with self.assertRaises(ValueError):
			self.udsclient.communication_control(control_type=-1, communication_type=valid_com_type)

		with self.assertRaises(ValueError):
			self.udsclient.communication_control(control_type=0, communication_type='x')


from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class TransferData(BaseService):
	_sid = 0x36
	_use_subfunction = False

	supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.RequestSequenceError,
							Response.Code.RequestOutOfRange,
							Response.Code.TransferDataSuspended,
							Response.Code.GeneralProgrammingFailure,
							Response.Code.WrongBlockSequenceCounter,
							Response.Code.VoltageTooHigh,
							Response.Code.VoltageTooLow
							]

	@classmethod
	def make_request(cls, sequence_number, data=None):
		"""
		Generates a request for TransferData

		:param sequence_number: Corresponds to an 8bit counter that should increment for each new block transferred.
			Allowed values are from 0 to 0xFF
		:type sequence_number: int

		:param data: Optional additional data to send to the server
		:type data: bytes

		:raises ValueError: If parameters are out of range, missing or wrong type
		"""		
		from udsoncan import Request, MemoryLocation
		
		ServiceHelper.validate_int(sequence_number, min=0, max=0xFF, name='Block sequence counter')	# Not a subfunction!

		if data is not None and not isinstance(data, bytes):
			raise ValueError('data must be a bytes object')

		request = Request(service=cls)
		request.data = struct.pack('B', sequence_number)

		if data is not None:
			request.data += data
		return request

	@classmethod
	def interpret_response(cls, response):
		"""
		Populates the response ``service_data`` property with an instance of :class:`TransferData.ResponseData<udsoncan.services.TransferData.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`

		:raises InvalidResponseException: If length of ``response.data`` is too short
		"""		
		if len(response.data) < 1:
			raise InvalidResponseException(response, "Response data must be at least 1 bytes")

		response.service_data = cls.ResponseData()
		response.service_data.sequence_number_echo = response.data[0]
		response.service_data.parameter_records = response.data[1:] if len(response.data) > 1 else b''

	class ResponseData(BaseResponseData):
		"""
		.. data:: sequence_number_echo

			Requests subfunction echoed back by the server

		.. data:: parameter_records

			Optional additional data associated with the response.
		"""		
		def __init__(self):
			super().__init__(TransferData)
			self.sequence_number_echo = None
			self.parameter_records = None



from udsoncan.client import Client
from udsoncan import services
from udsoncan.exceptions import *

from test.ClientServerTest import ClientServerTest

class TestTesterPresent(ClientServerTest):
	def __init__(self, *args, **kwargs):
		ClientServerTest.__init__(self, *args, **kwargs)
	
	def test_tester_present_success(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x3E\x00")
		self.conn.fromuserqueue.put(b"\x7E\x00")	# Positive response

	def _test_tester_present_success(self):
		response = self.udsclient.tester_present()
		self.assertTrue(response.positive)

	def test_tester_present_success_spr(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x3E\x80")
		self.conn.fromuserqueue.put('wait')	# Syncronize

	def _test_tester_present_success_spr(self):
		with self.udsclient.suppress_positive_response:
			response = self.udsclient.tester_present()
			self.assertEqual(response, None)
		self.conn.fromuserqueue.get(timeout=0.2)	#Avoid closing connection prematurely

	def test_tester_present_denied_exception(self):
		self.wait_request_and_respond(b"\x7F\x3E\x13") # IMLOIF

	def _test_tester_present_denied_exception(self):
		with self.assertRaises(NegativeResponseException) as handle:
			self.udsclient.tester_present()
		response = handle.exception.response

		self.assertTrue(response.valid)
		self.assertFalse(response.positive)
		self.assertTrue(issubclass(response.service, services.TesterPresent))
		self.assertEqual(response.code, 0x13)

	def test_tester_present_denied_no_exception(self):
		self.wait_request_and_respond(b"\x7F\x3E\x13") # IMLOIF

	def _test_tester_present_denied_no_exception(self):
		self.udsclient.config['exception_on_negative_response'] = False
		response = self.udsclient.tester_present()

		self.assertTrue(response.valid)
		self.assertFalse(response.positive)
		self.assertTrue(issubclass(response.service, services.TesterPresent))
		self.assertEqual(response.code, 0x13)

	def test_tester_present_invalidservice_exception(self):
		self.wait_request_and_respond(b"\x00\x00") #Inexistent Service

	def _test_tester_present_invalidservice_exception(self):
		with self.assertRaises(InvalidResponseException) as handle:
			self.udsclient.tester_present()

	def test_tester_present_invalidservice_no_exception(self):
		self.wait_request_and_respond(b"\x00\x00") #Inexistent Service

	def _test_tester_present_invalidservice_no_exception(self):
		self.udsclient.config['exception_on_invalid_response'] = False
		response = self.udsclient.tester_present()
		self.assertFalse(response.valid)

	def test_tester_present_wrongservice_exception(self):
		self.wait_request_and_respond(b"\x50\x00") # Valid but wrong service (Tester Present)

	def _test_tester_present_wrongservice_exception(self):
		with self.assertRaises(UnexpectedResponseException) as handle:
			self.udsclient.tester_present()

	def test_tester_present_wrongservice_no_exception(self):
		self.wait_request_and_respond(b"\x50\x00") # Valid but wrong service (Tester Present)

	def _test_tester_present_wrongservice_no_exception(self):
		self.udsclient.config['exception_on_unexpected_response'] = False
		response = self.udsclient.tester_present()
		self.assertTrue(response.valid)
		self.assertTrue(response.unexpected)


from test.UdsTest import UdsTest
from test.stub import StubbedIsoTPSocket
from udsoncan.exceptions import *
import socket

class TestStubbedIsoTPSocket(UdsTest):
	def test_open(self):
		tpsock = StubbedIsoTPSocket()
		self.assertFalse(tpsock.bound)
		tpsock.bind(interface='vcan0', rxid=0x100, txid=0x101)
		self.assertTrue(tpsock.bound)
		tpsock.close()
		self.assertFalse(tpsock.bound)

	def test_transmit(self):
		tpsock1 = StubbedIsoTPSocket()
		tpsock2 = StubbedIsoTPSocket(timeout=0.5)
		tpsock1.bind(interface='vcan0', rxid=0x200, txid=0x201)
		tpsock2.bind(interface='vcan0', rxid=0x201, txid=0x200)
		
		payload1 = b"\x01\x02\x03\x04"
		tpsock1.send(payload1)
		payload2 = tpsock2.recv()
		self.assertEqual(payload1, payload2)

	def test_multicast(self):
		tpsock1 = StubbedIsoTPSocket()
		tpsock2 = StubbedIsoTPSocket(timeout=0.5)
		tpsock3 = StubbedIsoTPSocket(timeout=0.5)
		tpsock1.bind(interface='vcan0', rxid=0x300, txid=0x301)
		tpsock2.bind(interface='vcan0', rxid=0x301, txid=0x300)
		tpsock3.bind(interface='vcan0', rxid=0x301, txid=0x300)
		
		payload1 = b"\x01\x02\x03\x04"
		tpsock1.send(payload1)
		payload2 = tpsock2.recv()
		payload3 = tpsock3.recv()
		self.assertEqual(payload1, payload2)
		self.assertEqual(payload1, payload3)

	def test_empty_on_close(self):
		tpsock1 = StubbedIsoTPSocket()
		tpsock2 = StubbedIsoTPSocket(timeout=0.2)
		tpsock1.bind(interface='vcan0', rxid=0x400, txid=0x401)
		tpsock2.bind(interface='vcan0', rxid=0x401, txid=0x400)
		
		payload = b"\x01\x02\x03\x04"
		tpsock1.send(payload)
		tpsock2.close()

		with self.assertRaises(socket.timeout):
			tpsock2.recv()

	def test_no_listener(self):
		tpsock1 = StubbedIsoTPSocket()
		tpsock2 = StubbedIsoTPSocket(timeout=0.2)
		tpsock1.bind(interface='vcan0', rxid=0x400, txid=0x401)
		
		payload = b"\x01\x02\x03\x04"
		tpsock1.send(payload)
		tpsock2.bind(interface='vcan0', rxid=0x401, txid=0x400)
		
		with self.assertRaises(socket.timeout):
			tpsock2.recv()
		


from udsoncan import Request, services
from test.UdsTest import UdsTest

class DummyServiceNormal(services.BaseService):
	_sid = 0x13

class DummyServiceNoSubunction(services.BaseService):
	_sid = 0x13
	_use_subfunction = False


class TestRequest(UdsTest):
	
	def test_create_from_instance_ok(self):
		req = Request(DummyServiceNormal())
		self.assertEqual(req.service.request_id(), 0x13)

	def test_create_from_class_ok(self):
		req = Request(DummyServiceNormal, subfunction=0x44)
		self.assertEqual(req.service.request_id(), 0x13)

	def test_make_payload_basic(self):
		req = Request(DummyServiceNormal, subfunction=0x44)
		payload = req.get_payload()
		self.assertEqual(b"\x13\x44", payload)

	def test_make_payload_custom_data(self):
		req = Request(DummyServiceNormal, subfunction=0x44)
		req.data = b"\x12\x34\x56\x78"
		payload = req.get_payload()
		self.assertEqual(b"\x13\x44\x12\x34\x56\x78", payload)
	
	def test_make_payload_custom_data_no_subfunction(self):
		req = Request(DummyServiceNoSubunction, subfunction=0x44)
		req.data = b"\x12\x34\x56\x78"
		payload = req.get_payload()
		self.assertEqual(b"\x13\x12\x34\x56\x78", payload)			

	def test_suppress_positive_response(self):
		req = Request(DummyServiceNormal, subfunction=0x44, suppress_positive_response=True)
		payload = req.get_payload()
		self.assertEqual(b"\x13\xC4", payload)	# Subfunction bit 7 is set

	def test_suppress_positive_response_override(self):
		req = Request(DummyServiceNormal, subfunction=0x44, suppress_positive_response=False)
		payload = req.get_payload(suppress_positive_response=True)
		self.assertEqual(b"\x13\xC4", payload)	# Subfunction bit 7 is set	

		req = Request(DummyServiceNormal, subfunction=0x44, suppress_positive_response=True)
		payload = req.get_payload(suppress_positive_response=False)
		self.assertEqual(b"\x13\x44", payload)	# Subfunction bit 7 is cleared		

	def test_from_payload_basic(self):
		payload=b'\x3E\x01'	# 0x3e = TesterPresent
		req = Request.from_payload(payload)
		self.assertEqual(req.service.request_id(), 0x3E)
		self.assertEqual(req.subfunction, 0x01)
		self.assertFalse(req.suppress_positive_response, 0x01)

	def test_from_payload_suppress_positive_response(self):
		payload=b'\x3E\x81'	# 0x3e = TesterPresent
		req = Request.from_payload(payload)
		self.assertEqual(req.service.request_id(), 0x3E)
		self.assertEqual(req.subfunction, 0x01)
		self.assertTrue(req.suppress_positive_response)

	def test_from_payload_custom_data(self):
		payload=b'\x3E\x01\x12\x34\x56\x78'	# 0x3E = TesterPresent
		req = Request.from_payload(payload)
		self.assertEqual(req.service.request_id(), 0x3E)
		self.assertEqual(req.subfunction, 0x01)
		self.assertEqual(req.data, b'\x12\x34\x56\x78')

	def test_from_empty_payload(self):
		payload = b''
		req = Request.from_payload(payload)
		self.assertIsNone(req.service)
		self.assertIsNone(req.subfunction)
		self.assertIsNone(req.data)

	def test_from_bad_payload(self):
		payload = b'\xFF\xFF'
		req = Request.from_payload(payload)
		self.assertIsNone(req.service)
		self.assertIsNone(req.subfunction)
		self.assertIsNone(req.data)

	def test_str_repr(self):
		req = Request(DummyServiceNormal)
		str(req)
		req.__repr__()

	def test_from_input_param(self):
		with self.assertRaises(ValueError):
			req = Request("a string")	

		with self.assertRaises(ValueError):
			req = Request(DummyServiceNormal, "string")	

		with self.assertRaises(ValueError):
			req = Request(DummyServiceNormal(), data=123)	

	def test_spr_with_no_subfunction(self):
		with self.assertRaises(ValueError):
			Request(service=DummyServiceNoSubunction, suppress_positive_response=True)




from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class RequestTransferExit(BaseService):
	_sid = 0x37
	_use_subfunction = False
	_no_response_data = True

	supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.RequestSequenceError
							]

	@classmethod
	def make_request(cls, data=None):
		"""
		Generates a request for RequestTransferExit

		:param data: Additional optional data to send to the server
		:type data: bytes

		:raises ValueError: If parameters are out of range, missing or wrong type
		"""			
		from udsoncan import Request, MemoryLocation
		
		if data is not None and not isinstance(data, bytes):
			raise ValueError('data must be a bytes object')

		request = Request(service=cls, data=data)
		return request

	@classmethod
	def interpret_response(cls, response):
		"""
		Populates the response ``service_data`` property with an instance of :class:`RequestTransferExit.ResponseData<udsoncan.services.RequestTransferExit.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`
		"""				
		response.service_data = cls.ResponseData()
		response.service_data.parameter_records = response.data

	class ResponseData(BaseResponseData):
		"""
		.. data:: parameter_records

			bytes object containing optional data provided by the server
		"""		
		def __init__(self):
			super().__init__(RequestTransferExit)
			self.parameter_records = None


import socket
import queue
import threading
import logging
import binascii
from abc import ABC, abstractmethod

from udsoncan.Request import Request
from udsoncan.Response import Response
from udsoncan.exceptions import TimeoutException

class BaseConnection(ABC):

	def __init__(self, name=None):
		if name is None:
			self.name = 'Connection'
		else:
			self.name = 'Connection[%s]' % (name)

		self.logger = logging.getLogger(self.name)

	def send(self, data):
		"""Sends data to the underlying transport protocol

		:param data: The data or object to send. If a Request or Response is given, the value returned by get_payload() will be sent.
		:type data: bytes, Request, Response

		:returns: None
		"""

		if isinstance(data, Request) or isinstance(data, Response):
			payload = data.get_payload()  
		else :
			payload = data

		self.logger.debug('Sending %d bytes : [%s]' % (len(payload), binascii.hexlify(payload) ))
		self.specific_send(payload)

	def wait_frame(self, timeout=2, exception=False):
		"""Waits for the reception of a frame of data from the underlying transport protocol

		:param timeout: The maximum amount of time to wait before giving up in seconds
		:type timeout: int
		:param exception: Boolean value indicating if this function may return exceptions.
			When ``True``, all exceptions may be raised, including ``TimeoutException``
			When ``False``, all exceptions will be logged as ``DEBUG`` and ``None`` will be returned.
		:type exception: bool

		:returns: Received data
		:rtype: bytes or None
		"""
		try:
			frame = self.specific_wait_frame(timeout=timeout)
		except Exception as e:
			self.logger.debug('No data received: [%s] - %s ' % (e.__class__.__name__, str(e)))

			if exception == True:
				raise
			else:
				frame = None

		if frame is not None:
			self.logger.debug('Received %d bytes : [%s]' % (len(frame), binascii.hexlify(frame) ))
		return frame
	
	def __enter__(self):
		return self

	@abstractmethod
	def specific_send(self, payload):
		"""The implementation of the send method.

		:param payload: Data to send
		:type payload: bytes
		
		:returns: None
		"""
		pass

	@abstractmethod
	def specific_wait_frame(self, timeout=2):
		"""The implementation of the ``wait_frame`` method. 

		:param timeout: The maximum amount of time to wait before giving up
		:type timeout: int
		
		:returns: Received data
		:rtype: bytes or None
		"""
		pass

	@abstractmethod
	def open(self):
		""" Set up the connection object. 

		:returns: None
		"""
		pass

	@abstractmethod
	def close(self):
		""" Close the connection object
		
		:returns: None
		"""
		pass	

	@abstractmethod
	def empty_rxqueue(self):
		""" Empty all unread data in the reception buffer.
		
		:returns: None
		"""
		pass
	
	def __exit__(self, type, value, traceback):
		pass


class SocketConnection(BaseConnection):
	"""
	Sends and receives data through a socket.

	:param sock: The socket to use. This socket must be bound and ready to use. Only ``send()`` and ``recv()`` will be called by this Connection
	:type sock: socket.socket
	:param bufsize: Maximum buffer size of the socket, this value is passed to ``recv()``
	:type bufsize: int
	:param name: This name is included in the logger name so that its output can be redirected. The logger name will be ``Connection[<name>]``
	:type name: string

	"""
	def __init__(self, sock, bufsize=4095, name=None):
		BaseConnection.__init__(self, name)

		self.rxqueue = queue.Queue()
		self.exit_requested = False
		self.opened = False
		self.rxthread = threading.Thread(target=self.rxthread_task)
		self.sock = sock
		self.sock.settimeout(0.1)	# for recv
		self.bufsize=bufsize


	def open(self):
		self.exit_requested = False
		self.rxthread.start()
		self.opened = True
		self.logger.info('Connection opened')
		return self

	def __enter__(self):
		return self

	def __exit__(self, type, value, traceback):
		self.close()

	def is_open(self):
		return self.opened

	def rxthread_task(self):
		while not self.exit_requested:
			try:
				data = self.sock.recv(self.bufsize)
				if data is not None:
					self.rxqueue.put(data)
			except socket.timeout:
				pass
			except Exception:
				self.exit_requested = True


	def close(self):
		self.exit_requested = True
		self.opened = False
		self.logger.info('Connection closed')

	def specific_send(self, payload):
		self.sock.send(payload)

	def specific_wait_frame(self, timeout=2):
		if not self.opened:
			raise RuntimeError("Connection is not open")

		timedout = False
		frame = None
		try:
			frame = self.rxqueue.get(block=True, timeout=timeout)

		except queue.Empty:
			timedout = True
			
		if timedout:
			raise TimeoutException("Did not received frame in time (timeout=%s sec)" % timeout)

		return frame

	def empty_rxqueue(self):
		while not self.rxqueue.empty():
			self.rxqueue.get()


class IsoTPConnection(BaseConnection):
	"""
	Sends and receives data through an ISO-TP socket. Makes cleaner code than SocketConnection but offers no additional functionality.
	The `isotp module <https://github.com/pylessard/python-can-isotp>`_ must be installed in order to use this connection

	:param interface: The can interface to use (example: `can0`)
	:type interface: string
	:param rxid: The reception CAN id
	:type rxid: int 
	:param txid: The transmission CAN id
	:type txid: int
	:param name: This name is included in the logger name so that its output can be redirected. The logger name will be ``Connection[<name>]``
	:type name: string
	:param tpsock: An optional ISO-TP socket to use instead of creating one.
	:type tpsock: isotp.socket

	"""
	def __init__(self, interface, rxid, txid, name=None, tpsock=None):
		
		BaseConnection.__init__(self, name)

		self.interface=interface
		self.rxid=rxid
		self.txid=txid
		self.rxqueue = queue.Queue()
		self.exit_requested = False
		self.opened = False

		self.rxthread = threading.Thread(target=self.rxthread_task)
		if tpsock is None:
			import isotp
			self.tpsock = isotp.socket(timeout=0.1)
		else:
			self.tpsock = tpsock


	def open(self):
		self.tpsock.bind(self.interface, rxid=self.rxid, txid=self.txid)
		self.exit_requested = False
		self.rxthread.start()
		self.opened = True
		self.logger.info('Connection opened')
		return self

	def __enter__(self):
		return self

	def __exit__(self, type, value, traceback):
		self.close()

	def is_open(self):
		return self.tpsock.bound

	def rxthread_task(self):
		while not self.exit_requested:
			try:
				data = self.tpsock.recv()
				if data is not None:
					self.rxqueue.put(data)
			except socket.timeout:
				pass
			except Exception:
				self.exit_requested = True


	def close(self):
		self.exit_requested = True
		self.tpsock.close()
		self.opened = False
		self.logger.info('Connection closed')

	def specific_send(self, payload):
		self.tpsock.send(payload)

	def specific_wait_frame(self, timeout=2):
		if not self.opened:
			raise RuntimeError("Connection is not open")

		timedout = False
		frame = None
		try:
			frame = self.rxqueue.get(block=True, timeout=timeout)

		except queue.Empty:
			timedout = True
			
		if timedout:
			raise TimeoutException("Did not received ISOTP frame in time (timeout=%s sec)" % timeout)

		return frame

	def empty_rxqueue(self):
		while not self.rxqueue.empty():
			self.rxqueue.get()


class QueueConnection(BaseConnection):
	"""
	Sends and receives data using 2 Python native queues.

	- ``MyConnection.fromuserqueue`` : Data read from this queue when ``wait_frame`` is called
	- ``MyConnection.touserqueue`` : Data written to this queue when ``send`` is called

	:param mtu: Optional maximum frame size. Messages will be truncated to this size
	:type mtu: int
	:param name: This name is included in the logger name so that its output can be redirected. The logger name will be ``Connection[<name>]``
	:type name: string

	"""
	def __init__(self, name=None, mtu=4095):
		BaseConnection.__init__(self, name)

		self.fromuserqueue = queue.Queue()	# Client reads from this queue. Other end is simulated
		self.touserqueue = queue.Queue()	# Client writes to this queue. Other end is simulated
		self.opened = False
		self.mtu = mtu

	def open(self):
		self.opened = True
		self.logger.info('Connection opened')
		return self

	def __enter__(self):
		return self

	def __exit__(self, type, value, traceback):
		self.close()

	def is_open(self):
		return self.opened 

	def close(self):
		self.empty_rxqueue()
		self.empty_txqueue()
		self.opened = False
		self.logger.info('Connection closed')	

	def specific_send(self, payload):
		if self.mtu is not None:
			if len(payload) > self.mtu:
				self.logger.warning("Truncating payload to be set to a length of %d" % (self.mtu))
				payload = payload[0:self.mtu]

		self.touserqueue.put(payload)

	def specific_wait_frame(self, timeout=2):
		if not self.opened:
			raise RuntimeException("Connection is not open")

		timedout = False
		frame = None
		try:
			frame = self.fromuserqueue.get(block=True, timeout=timeout)
		except queue.Empty:
			timedout = True
			
		if timedout:
			raise TimeoutException("Did not receive frame from user queue in time (timeout=%s sec)" % timeout)
		
		if self.mtu is not None:
			if frame is not None and len(frame) > self.mtu:
				self.logger.warning("Truncating received payload to a length of %d" % (self.mtu))
				frame = frame[0:self.mtu]

		return frame

	def empty_rxqueue(self):
		while not self.fromuserqueue.empty():
			self.fromuserqueue.get()

	def empty_txqueue(self):
		while not self.touserqueue.empty():
			self.touserqueue.get()




from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct, math

class InputOutputControlByIdentifier(BaseService):
	_sid = 0x2F
	_use_subfunction = False

	#
	class ControlParam(BaseSubfunction):
		"""
		InputOutputControlByIdentifier defined control parameters as defined by ISO-14229:2006, Annex E
		"""	

		__pretty_name__ = 'control parameter'

		returnControlToECU = 0
		resetToDefault = 1
		freezeCurrentState = 2
		shortTermAdjustment = 3

	supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.ConditionsNotCorrect,
							Response.Code.RequestOutOfRange,
							Response.Code.SecurityAccessDenied
							]

	@classmethod
	def make_request(cls, did, control_param=None, values=None, masks=None, ioconfig=None):
		"""
		Generates a request for InputOutputControlByIdentifier

		:param did: Data identifier to represent the IO
		:type did: int

		:param control_param: Optional parameter that can be a value defined in :class:`InputOutputControlByIdentifier.ControlParam<udsoncan.services.InputOutputControlByIdentifier.ControlParam>`
		:type control_param: int

		:param values: Optional values to send to the server. This parameter will be given to :ref:`DidCodec<DidCodec>`.encode() method. 
			It can be:
			
				- A list for positional arguments
				- A dict for named arguments
				- An instance of :ref:`IOValues<IOValues>` for mixed arguments

		:type values: list, dict, :ref:`IOValues<IOValues>`

		:param masks: Optional mask record for composite values. The mask definition must be included in ``ioconfig``
			It can be:

				- A list naming the bit mask to set
				- A dict with the mask name as a key and a boolean as the value (True to set the mask, False to clear it)
				- An instance of :ref:`IOMask<IOMask>`
				- A boolean value to set all masks to the same value.
		:type masks: list, dict, :ref:`IOMask<IOMask>`, bool

		:param ioconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec<DidCodec>` class or pack/unpack string. 
			It is possible to use composite :ref:`DidCodec<DidCodec>` by specifying a dict with entries : codec, mask, mask_size.
		:type ioconfig: dict[int] = :ref:`DidCodec<DidCodec>`, dict

		:raises ValueError: If parameters are out of range, missing or wrong type
		:raises ConfigError: If given DID is not defined within ioconfig
		"""	

		from udsoncan import Request, IOMasks, IOValues, DidCodec
		
		ServiceHelper.validate_int(did, min=0, max=0xffff, name='DID')
		if control_param is not None:
			if not isinstance(control_param, int):
				raise ValueError("control_param  must be a valid integer")

			if control_param < 0 or control_param > 3:
				raise ValueError('control_param must either be returnControlToECU(0), resetToDefault(1), freezeCurrentState(2), shortTermAdjustment(3). %d given.' % control_param)
		
		if values is not None:
			if isinstance(values, list):
				values = IOValues(*values)
			if isinstance(values, dict):
				values = IOValues(**values)

			if not isinstance(values, IOValues):
				raise ValueError("values must be an instance of IOValues")

		if masks is not None:
			if isinstance(masks, list):
				masks = IOMasks(*masks)
			if isinstance(masks, dict):
				masks = IOMasks(**masks)

			if not isinstance(masks, IOMasks) and not isinstance(masks, bool):
				raise ValueError("masks must be an instance of IOMask or a boolean value")

		if values is None and masks is not None:
			raise ValueError('An IOValue must be given if a IOMask is provided.')	

		request = Request(service=cls)		

		request.data = b''
		ioconfig = ServiceHelper.check_io_config(did, ioconfig)	# IO dids are defined in client config.
		request.data += struct.pack('>H', did)

		# This parameter is optional according to standard
		if control_param is not None:
			request.data += struct.pack('B', control_param)
		
		codec = DidCodec.from_config(ioconfig[did])	# Get IO codec from config
		
		if values is not None:
			request.data += codec.encode(*values.args, **values.kwargs)

		if masks is not None: # Skip the masks byte if none is given.
			if isinstance(masks, bool):
				byte = b'\xFF' if masks == True else b'\x00'
				if 'mask_size' in  ioconfig[did]:
					request.data += (byte * ioconfig[did]['mask_size'])
				else:
					raise ConfigError('mask_size', msg='Given mask is boolean value, indicating that all mask should be set to same value, but no mask_size is defined in configuration. Cannot guess how many bits to set.')

			elif isinstance(masks, IOMasks):
				if 'mask' not in ioconfig[did]:
					raise ConfigError('mask', msg='Cannot apply given mask. Input/Output configuration does not define their position (and size).')
				masks_config = ioconfig[did]['mask']
				given_masks = masks.get_dict()

				numeric_val = 0
				for mask_name in given_masks:
					if mask_name not in masks_config:
						raise ConfigError('mask_size', msg='Cannot set mask bit for mask %s. The configuration does not define its position' % (mask_name))	
					
					if given_masks[mask_name] == True:
						numeric_val |= masks_config[mask_name]

				minsize = math.ceil(math.log(numeric_val+1, 2)/8.0)
				size = minsize if 'mask_size' not in ioconfig[did] else ioconfig[did]['mask_size']
				request.data += numeric_val.to_bytes(size, 'big')
		return request


	@classmethod
	def interpret_response(cls, response, control_param=None, tolerate_zero_padding=True, ioconfig=None):
		"""
		Populates the response ``service_data`` property with an instance of :class:`InputOutputControlByIdentifier.ResponseData<udsoncan.services.InputOutputControlByIdentifier.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`
		
		:param control_param:  Same optional control parameter value given to make_request()
		:type control_param: int
		
		:param tolerate_zero_padding: Ignore trailing zeros in the response data avoiding raising false :class:`InvalidResponseException<udsoncan.exceptions.InvalidResponseException>`.
		:type tolerate_zero_padding: bool

		:param ioconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec<DidCodec>` class or pack/unpack string. 
			It is possible to use composite DidCodec by specifying a dict with entries : codec, mask, mask_size.
		:type ioconfig: dict[int] = :ref:`DidCodec<DidCodec>`, dict

		:raises ValueError: If parameters are out of range, missing or wrong type
		:raises ConfigError: If DID echoed back by the server is not in the ``ioconfig`` definition
		:raises InvalidResponseException: If response data is incomplete or if DID data does not match codec length.
		"""	

		from udsoncan import DidCodec
		min_response_size = 2 if control_param is not None else 1	# Spec specifies that if first byte is a ControlParameter, it must be echoed back by the server

		if len(response.data) < min_response_size:
			raise InvalidResponseException(response, "Response must be at least %d bytes long" % min_response_size)

		response.service_data = cls.ResponseData()
		response.service_data.did_echo = struct.unpack(">H", response.data[0:2])[0]

		did = response.service_data.did_echo
		ioconfig = ServiceHelper.check_io_config(did, ioconfig)	# IO DIDs are defined in client config.
		codec = DidCodec.from_config(ioconfig[did])	# Get IO codec from config

		next_byte = 2
		if control_param is not None:
			if len(response.data) < next_byte:
				raise InvalidResponseException(response, 'Response should include an echo of the InputOutputControlParameter (0x%02x)' % control_param)
			response.service_data.control_param_echo = response.data[next_byte]
			next_byte +=1

		if len(response.data) >= next_byte:
			remaining_data = response.data[next_byte:]
			
			if len(remaining_data) > len(codec):
				if remaining_data[len(codec):] == b'\x00' * (len(remaining_data) - len(codec)):
					if tolerate_zero_padding:
						remaining_data = remaining_data[0:len(codec)]
			try:
				response.service_data.decoded_data = codec.decode(remaining_data)
			except Exception as e:
				raise InvalidResponseException(response, 'Response from server could not be decoded. Exception is : %s' % e)
			
	class ResponseData(BaseResponseData):
		"""
		.. data:: did_echo

			DID echoed back by the server

		.. data:: control_param_echo

			control_param echoed back by the server

		.. data:: decoded_data

			Value processed by the :ref:`DidCodec<DidCodec>`.decode() method
		"""			
		def __init__(self):
			super().__init__(InputOutputControlByIdentifier)
			self.did_echo = None
			self.control_param_echo = None
			self.decoded_data = None
			


from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class RoutineControl(BaseService):
	_sid = 0x31

	class ControlType(BaseSubfunction):
		"""
		RoutineControl defined subfunctions
		"""		
		__pretty_name__ = 'control type'

		startRoutine = 1
		stopRoutine = 2
		requestRoutineResults = 3

	supported_negative_response = [	 Response.Code.SubFunctionNotSupported,
							Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.ConditionsNotCorrect,
							Response.Code.RequestSequenceError,
							Response.Code.RequestOutOfRange,
							Response.Code.SecurityAccessDenied,
							Response.Code.GeneralProgrammingFailure
							]

	@classmethod
	def make_request(cls, routine_id, control_type, data=None):
		"""
		Generates a request for RoutineControl

		:param routine_id: The routine ID. Value should be between 0 and 0xFFFF
		:type routine_id: int

		:param control_type: Service subfunction. Allowed values are from 0 to 0x7F
		:type control_type: bytes

		:param data: Optional additional data to provide to the server
		:type data: bytes

		:raises ValueError: If parameters are out of range, missing or wrong type
		"""		
		from udsoncan import Request

		ServiceHelper.validate_int(routine_id, min=0, max=0xFFFF, name='Routine ID')
		ServiceHelper.validate_int(control_type, min=0, max=0x7F, name='Routine control type')
		
		if data is not None:
			if not isinstance(data, bytes):
				raise ValueError('data must be a valid bytes object')

		request = Request(service=cls, subfunction=control_type)
		request.data = struct.pack('>H', routine_id)
		if data is not None:
			request.data += data

		return request

	@classmethod
	def interpret_response(cls, response):
		"""
		Populates the response ``service_data`` property with an instance of :class:`RoutineControl.ResponseData<udsoncan.services.RoutineControl.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`

		:raises InvalidResponseException: If length of ``response.data`` is too short
		"""

		if len(response.data) < 3: 	
			raise InvalidResponseException(response, "Response data must be at least 3 bytes")

		response.service_data = cls.ResponseData()
		response.service_data.control_type_echo = response.data[0]
		response.service_data.routine_id_echo = struct.unpack(">H", response.data[1:3])[0]
		response.service_data.routine_status_record = response.data[3:] if len(response.data) >3 else b''

	class ResponseData(BaseResponseData):
		"""
		.. data:: control_type_echo

			Requests subfunction echoed back by the server

		.. data:: routine_id_echo

			Requests routine ID echoed back by the server.

		.. data:: routine_status_record

			Additional data associated with the response.
		"""		
		def __init__(self):
			super().__init__(RoutineControl)

			self.control_type_echo = None
			self.routine_id_echo = None
			self.routine_status_record = None
			


from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class DiagnosticSessionControl(BaseService):
	_sid = 0x10

	supported_negative_response = [	Response.Code.SubFunctionNotSupported, 
									Response.Code.IncorrectMessageLegthOrInvalidFormat,
									Response.Code.ConditionsNotCorrect
									]
	class Session(BaseSubfunction):
		"""
		DiagnosticSessionControl defined subfunctions
		"""		
		__pretty_name__ = 'session'	

		defaultSession = 1
		programmingSession = 2
		extendedDiagnosticSession = 3
		safetySystemDiagnosticSession = 4

	@classmethod
	def make_request(cls, session):
		"""
		Generates a request for DiagnosticSessionControl service

		:param session: Service subfunction. Allowed values are from 0 to 0x7F
		:type session: int

		:raises ValueError: If parameters are out of range, missing or wrong type
		"""

		from udsoncan import Request
		ServiceHelper.validate_int(session, min=0, max=0x7F, name='Session number')
		return Request(service=cls, subfunction=session)

	@classmethod
	def interpret_response(cls, response):
		"""
		Populates the response ``service_data`` property with an instance of :class:`DiagnosticSessionControl.ResponseData<udsoncan.services.DiagnosticSessionControl.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`

		:raises InvalidResponseException: If length of ``response.data`` is too short
		"""

		if len(response.data) < 1: 	# Should not happen as response decoder will raise an exception.
			raise InvalidResponseException(response, "Response data must be at least 1 bytes")

		response.service_data = cls.ResponseData()
		response.service_data.session_echo = response.data[0]
		response.service_data.session_param_records = response.data[1:] if len(response.data) > 1 else b''

		return response

	class ResponseData(BaseResponseData):
		"""
		.. data:: session_echo

			Request subfunction echoed back by the server

		.. data:: session_param_records

			Additional data associated with the response.
		"""		
		def __init__(self):
			super().__init__(DiagnosticSessionControl)
			self.session_echo = None
			self.session_param_records = None


from udsoncan.client import Client
from udsoncan import services
from udsoncan.exceptions import *

from test.ClientServerTest import ClientServerTest

class TestDiagnosticSessionControl(ClientServerTest):
	def __init__(self, *args, **kwargs):
		ClientServerTest.__init__(self, *args, **kwargs)

	def test_dsc_success(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x10\x01")
		self.conn.fromuserqueue.put(b"\x50\x01\x99\x88")	# Positive response

	def _test_dsc_success(self):
		response = self.udsclient.change_session(services.DiagnosticSessionControl.Session.defaultSession)
		self.assertEqual(response.service_data.session_echo, 1)
		self.assertEqual(response.service_data.session_param_records, b"\x99\x88")

	def test_dsc_success_spr(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x10\x81")
		self.conn.fromuserqueue.put("wait")	#Synchronize

	def _test_dsc_success_spr(self):
		with self.udsclient.suppress_positive_response:
			response = self.udsclient.change_session(services.DiagnosticSessionControl.Session.defaultSession)
			self.assertEqual(response, None)
		self.conn.fromuserqueue.get(timeout=0.2)	#Avoid closing connection prematurely

	def test_dsc_denied_exception(self):
		request = self.conn.touserqueue.get(timeout=0.2)
		self.assertEqual(request, b"\x10\x08")
		self.conn.fromuserqueue.put(b"\x7F\x10\x12") # Subfunction not supported

	def _test_dsc_denied_exception(self):
		with self.assertRaises(NegativeResponseException) as handle:
			self.udsclient.change_session(0x08)
		response = handle.exception.response

		self.assertTrue(response.valid)
		self.assertTrue(issubclass(response.service, services.DiagnosticSessionControl))
		self.assertEqual(response.code, 0x12)

	def test_dsc_denied_no_exception(self):
		self.wait_request_and_respond(b"\x7F\x10\x12") # Subfunction not supported

	def _test_dsc_denied_no_exception(self):
		self.udsclient.config['exception_on_negative_response'] = False
		response = self.udsclient.change_session(0x08)
		self.assertTrue(response.valid)
		self.assertFalse(response.positive)

	def test_dsc_bad_subfunction_exception(self):
		self.wait_request_and_respond(b"\x50\x02")	# Positive response

	def _test_dsc_bad_subfunction_exception(self):
		with self.assertRaises(UnexpectedResponseException):
			self.udsclient.change_session(services.DiagnosticSessionControl.Session.defaultSession)

	def test_dsc_bad_subfunction_no_exception(self):
		self.wait_request_and_respond(b"\x50\x02")	# Positive response

	def _test_dsc_bad_subfunction_no_exception(self):
		self.udsclient.config['exception_on_unexpected_response'] = False
		response = self.udsclient.change_session(services.DiagnosticSessionControl.Session.defaultSession)
		self.assertTrue(response.valid)
		self.assertTrue(response.unexpected)

	def test_dsc_invalidservice_exception(self):
		self.wait_request_and_respond(b"\x00\x02") #Inexistent Service

	def _test_dsc_invalidservice_exception(self):
		with self.assertRaises(InvalidResponseException) as handle:
			self.udsclient.change_session(0x02)

	def test_dsc_invalidservice_no_exception(self):
		self.wait_request_and_respond(b"\x00\x02") #Inexistent Service

	def _test_dsc_invalidservice_no_exception(self):
		self.udsclient.config['exception_on_invalid_response'] = False
		response = self.udsclient.change_session(0x02)
		self.assertFalse(response.valid)

	def test_ecu_reset_wrongservice_exception(self):
		self.udsclient.config['exception_on_invalid_response'] = False
		self.wait_request_and_respond(b"\x7E\x00") # Valid but wrong service (Tester Present)

	def _test_ecu_reset_wrongservice_exception(self):
		with self.assertRaises(UnexpectedResponseException) as handle:
			self.udsclient.change_session(0x55)

	def test_ecu_reset_wrongservice_no_exception(self):
		self.wait_request_and_respond(b"\x7E\x00") # Valid but wrong service (Tester Present)

	def _test_ecu_reset_wrongservice_no_exception(self):
		self.udsclient.config['exception_on_unexpected_response'] = False
		response = self.udsclient.change_session(0x55)
		self.assertTrue(response.valid)
		self.assertTrue(response.unexpected)

	def test_bad_param(self):
		pass

	def _test_bad_param(self):
		with self.assertRaises(ValueError):
			success = self.udsclient.change_session(0x100)

		with self.assertRaises(ValueError):
			success = self.udsclient.change_session(-1)



from udsoncan import Response, Request, services, DidCodec, Routine, IOMasks, Dtc, DataIdentifier, MemoryLocation
from udsoncan.exceptions import *
from udsoncan.configs import default_client_config
import struct
import logging
import math
import binascii
import traceback
import functools

class Client:
	"""
	__init__(self, conn, config=default_client_config, request_timeout = 1)

	Object that interacts with a UDS server. 
	It builds a service request, sends it to the server, receives and parses its response, detects communication anomalies and logs what it is doing for further debugging.

	:param conn: The underlying protocol interface.
	:type conn: :ref:`Connection<Connection>`

	:param config: The :ref:`client configuration<client_config>`
	:type config: dict

	:param request_timeout: Maximum amount of time to wait for a response after sending a request (in seconds). After this time is elapsed, a TimeoutException will be raised.
	:type request_timeout: int
	"""

	class SuppressPositiveResponse:
		def __init__(self):
			self.enabled = False
		
		def __enter__(self):
			self.enabled = True
			return self

		def __exit__(self, type, value, traceback):
			self.enabled = False

	def __init__(self, conn, config=default_client_config, request_timeout = 1):
		self.conn = conn
		self.request_timeout = request_timeout
		self.config = dict(config) # Makes a copy of given configuration
		self.suppress_positive_response = Client.SuppressPositiveResponse()

		self.refresh_config()

	def __enter__(self):
		self.open()
		return self
	
	def __exit__(self, type, value, traceback):
		self.close()

	def open(self):
		if not self.conn.is_open():
			self.conn.open()

	def close(self):
		self.conn.close()

	def configure_logger(self):
		logger_name = 'UdsClient'
		if 'logger_name' in self.config:
			logger_name = "UdsClient[%s]" % self.config['logger_name']

		self.logger = logging.getLogger(logger_name)

	def set_config(self, key, value):
		self.config[key] = value
		self.refresh_config()

	def refresh_config(self):
		self.configure_logger()


	
	# Decorator to apply on functions that the user will call.
	# Each function raises exceptions. This decorator handles these exceptions, logs them, 
	# then suppresses them or not depending on the client configuration.
	# if func1 and func2 are decorated and func2 calls func1, it should be done this way : self.func1._func_no_error_management(self, ...)
	
	def standard_error_management(func):
		@functools.wraps(func)
		def decorated(self, *args, **kwargs):
			try:
				return func(self, *args, **kwargs)
			
			except NegativeResponseException as e:
				e.positive = False
				if self.config['exception_on_negative_response']:
					logline = '[%s] : %s' % (e.__class__.__name__, str(e))
					self.logger.warning(logline)
					raise
				else:
					self.logger.warning(str(e))
					return e.response
			
			except InvalidResponseException as e:
				e.response.valid = False
				if self.config['exception_on_invalid_response']:
					self.logger.error('[%s] : %s' % (e.__class__.__name__, str(e)))
					raise
				else:
					self.logger.error(str(e))
					return e.response
			
			except UnexpectedResponseException as e:
				e.response.unexpected = True
				if self.config['exception_on_unexpected_response']:
					self.logger.error('[%s] : %s' % (e.__class__.__name__, str(e)))
					raise
				else:
					self.logger.error(str(e))
					return e.response
			
			except Exception as e:
				self.logger.error('[%s] : %s' % (e.__class__.__name__, str(e)))
				raise

		decorated._func_no_error_management = func
		return decorated

	@standard_error_management
	def change_session(self, newsession):
		""" 
		Requests the server to change the diagnostic session with a :ref:`DiagnosticSessionControl<DiagnosticSessionControl>` service request

		:dependent configuration: ``exception_on_<type>_response``

		:param newsession: The session to try to switch. Values from :class:`DiagnosticSessionControl.Session <udsoncan.services.DiagnosticSessionControl.Session>` can be used.
		:type newsession: int 

		:return: The server response parsed by :meth:`DiagnosticSessionControl.interpret_response<udsoncan.services.DiagnosticSessionControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		req = services.DiagnosticSessionControl.make_request(newsession)

		named_newsession = '%s (0x%02x)' % (services.DiagnosticSessionControl.Session.get_name(newsession), newsession)
		self.logger.info('Switching session to : %s' % (named_newsession))
		
		response = self.send_request(req)
		if response is None:
			return 

		services.DiagnosticSessionControl.interpret_response(response)

		if newsession != response.service_data.session_echo:
			raise UnexpectedResponseException(response, "Response subfunction received from server (0x%02x) does not match the requested subfunction (0x%02x)" % (response.service_data.session_echo, newsession))

		return response

	@standard_error_management
	def request_seed(self, level):
		""" 
		Requests a seed to unlock a security level with the :ref:`SecurityAccess<SecurityAccess>` service 

		:dependent configuration: ``exception_on_<type>_response``

		:param level: The security level to unlock. If value is even, it will be converted to the corresponding odd value
		:type level: int 

		:return: The server response parsed by :meth:`SecurityAccess.interpret_response<udsoncan.services.SecurityAccess.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""		
		req = services.SecurityAccess.make_request(level, mode=services.SecurityAccess.Mode.RequestSeed)

		self.logger.info('Requesting seed to unlock security access level 0x%02x' % (req.subfunction))	# level may be corrected by service.
		
		response = self.send_request(req)
		if response is None:
			return

		services.SecurityAccess.interpret_response(response, mode=services.SecurityAccess.Mode.RequestSeed)

		expected_level = services.SecurityAccess.normalize_level(mode=services.SecurityAccess.Mode.RequestSeed, level=level)
		received_level = response.service_data.security_level_echo
		if expected_level != received_level:
			raise UnexpectedResponseException(response, "Response subfunction received from server (0x%02x) does not match the requested subfunction (0x%02x)" % (received_level, expected_level))

		self.logger.debug('Received seed : [%s]' % (binascii.hexlify(response.service_data.seed)))
		return response

	# Performs a SecurityAccess service request. Send key
	@standard_error_management
	def send_key(self, level, key):
		""" 
		Sends a key to unlock a security level with the :ref:`SecurityAccess<SecurityAccess>` service 

		:dependent configuration: ``exception_on_<type>_response``

		:param level: The security level to unlock. If value is odd, it will be converted to the corresponding even value
		:type level: int 

		:param key: The key to send to the server
		:type key: bytes 

		:return: The server response parsed by :meth:`SecurityAccess.interpret_response<udsoncan.services.SecurityAccess.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""			
		req = services.SecurityAccess.make_request(level, mode=services.SecurityAccess.Mode.SendKey, key=key)
		self.logger.info('Sending key to unlock security access level 0x%02x' % (req.subfunction))
		self.logger.debug('Key to send : [%s]' % (binascii.hexlify(key)))

		response = self.send_request(req)
		if response is None:
			return

		services.SecurityAccess.interpret_response(response, mode=services.SecurityAccess.Mode.SendKey)

		expected_level = services.SecurityAccess.normalize_level(mode=services.SecurityAccess.Mode.SendKey, level=level)
		received_level = response.service_data.security_level_echo
		if expected_level != received_level:
			raise UnexpectedResponseException(response, "Response subfunction received from server (0x%02x) does not match the requested subfunction (0x%02x)" % (received_level, expected_level))

		return response
	
	@standard_error_management
	def unlock_security_access(self, level):
		"""
		Successively calls request_seed and send_key to unlock a security level with the :ref:`SecurityAccess<SecurityAccess>` service.
		The key computation is done by calling config['security_algo']

		:dependent configuration: ``exception_on_<type>_response`` ``security_algo`` ``security_algo_params``

		:param level: The level to unlock. Can be the odd or even variant of it.
		:type level: int

		:return: The server response parsed by :meth:`SecurityAccess.interpret_response<udsoncan.services.SecurityAccess.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""

		if 'security_algo' not in self.config or not callable(self.config['security_algo']):
			raise NotImplementedError("Client configuration does not provide a security algorithm")
		
		seed = self.request_seed._func_no_error_management(self, level).service_data.seed
		params = self.config['security_algo_params'] if 'security_algo_params' in self.config else None
		key = self.config['security_algo'].__call__(seed, params)
		return self.send_key._func_no_error_management(self, level, key)

	
	@standard_error_management
	def tester_present(self):
		"""
		Sends a TesterPresent request to keep the session active.

		:dependent configuration: ``exception_on_<type>_response``

		:return: The server response parsed by :meth:`TesterPresent.interpret_response<udsoncan.services.TesterPresent.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		req = services.TesterPresent.make_request()

		self.logger.info('Sending TesterPresent request')
		response = self.send_request(req)
		if response is None:
			return

		services.TesterPresent.interpret_response(response)

		if req.subfunction != response.service_data.subfunction_echo:
			raise UnexpectedResponseException(response, "Response subfunction received from server (0x%02x) does not match the requested subfunction (0x%02x)" % (response.service_data.subfunction_echo, req.subfunction))

		return response

	@standard_error_management
	def read_data_by_identifier_first(self, didlist):
		"""
		Shortcut to extract a single DID. 
		Calls read_data_by_identifier then returns the first DID asked for. 

		:dependent configuration: ``exception_on_<type>_response`` ``data_identifiers`` ``tolerate_zero_padding``

		:param didlist: The list of DID to be read
		:type didlist: list[int]

		:return: The server response parsed by :meth:`ReadDataByIdentifier.interpret_response<udsoncan.services.ReadDataByIdentifier.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		values = self.read_data_by_identifier(didlist)
		if len(values) > 0 and len(didlist) > 0:
			return values[didlist[0]]

	@standard_error_management
	def read_data_by_identifier(self, didlist):
		"""
		Requests a value associated with a data identifier (DID) through the :ref:`ReadDataByIdentifier<ReadDataByIdentifier>` service.
		
		:dependent configuration: ``exception_on_<type>_response`` ``data_identifiers`` ``tolerate_zero_padding``

		See :ref:`an example<reading_a_did>` about how to read a DID

		:param didlist: The list of DID to be read
		:type didlist: list[int]

		:return: The server response parsed by :meth:`ReadDataByIdentifier.interpret_response<udsoncan.services.ReadDataByIdentifier.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		didlist = services.ReadDataByIdentifier.validate_didlist_input(didlist)
		req = services.ReadDataByIdentifier.make_request(didlist=didlist, didconfig=self.config['data_identifiers'])

		if len(didlist) == 1:
			self.logger.info("Reading data identifier : %s (%s)" % (didlist[0], DataIdentifier.name_from_id(didlist[0])))
		else:
			self.logger.info("Reading %d data identifier : %s" % (len(didlist), didlist))
		
		if 'data_identifiers' not in self.config or  not isinstance(self.config['data_identifiers'], dict):
			raise AttributeError('Configuration does not contains a valid data identifier description.')

		response = self.send_request(req)
		if response is None:
			return
		
		params = {
			'didlist' : didlist, 
			'didconfig' : self.config['data_identifiers'],
			'tolerate_zero_padding' : self.config['tolerate_zero_padding']
			}
		
		try:
			services.ReadDataByIdentifier.interpret_response(response, **params)
		except ConfigError as e:
			if e.key in didlist:
				raise
			else:
				raise UnexpectedResponseException(response, "Server returned values for data identifier 0x%04x that was not requested and no Codec was defined for it. Parsing must be stopped." % (e.key))
		
		set_request_didlist = set(didlist)
		set_response_didlist = set(response.service_data.values.keys())
		extra_did  = set_response_didlist - set_request_didlist
		missing_did  = set_request_didlist - set_response_didlist

		if len(extra_did) > 0:
			raise UnexpectedResponseException(response, "Server returned values for %d data identifier that were not requested. Dids are : %s" % (len(extra_did), extra_did))

		if len(missing_did) > 0:
			raise UnexpectedResponseException(response, "%d data identifier values are missing from server response. Dids are : %s" % (len(missing_did), missing_did))

		return response
		

	# Performs a WriteDataByIdentifier request.
	@standard_error_management
	def write_data_by_identifier(self, did, value):
		"""
		Requests to write a value associated with a data identifier (DID) through the :ref:`WriteDataByIdentifier<WriteDataByIdentifier>` service.
		
		:dependent configuration:  ``exception_on_<type>_response`` ``data_identifiers``

		:param didlist: The DID to write its value
		:type didlist: int
		
		:param value: Value given to the :ref:`DidCodec <DidCodec>`.encode method. The payload returned by the codec will be sent to the server.
		:type value: int

		:return: The server response parsed by :meth:`WriteDataByIdentifier.interpret_response<udsoncan.services.WriteDataByIdentifier.interpret_response>`
		:rtype: :ref:`Response<Response>`

		"""
		req = services.WriteDataByIdentifier.make_request(did, value, didconfig=self.config['data_identifiers'])
		self.logger.info("Writing data identifier %s (%s)", did, DataIdentifier.name_from_id(did))
		
		response = self.send_request(req)
		if response is None:
			return
		services.WriteDataByIdentifier.interpret_response(response)

		if response.service_data.did_echo != did:
			raise UnexpectedResponseException(response, "Server returned a response for data identifier 0x%02x while client requested for did 0x%02x" % (response.service_data.did_echo, did))
		
		return response

	@standard_error_management
	def ecu_reset(self, reset_type):
		"""
		Requests the server to execute a reset sequence through the :ref:`ECUReset<ECUReset>` service.

		:dependent configuration: ``exception_on_<type>_response``

		:param reset_type: The type of reset to perform.  :class:`ECUReset.ResetType<udsoncan.services.ECUReset.ResetType>`
		:type reset_type: int

		:return: The server response parsed by :meth:`ECUReset.interpret_response<udsoncan.services.ECUReset.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		req = services.ECUReset.make_request(reset_type)
		self.logger.info("Requesting ECU reset of type 0x%02x (%s)" % (reset_type, services.ECUReset.ResetType.get_name(reset_type)))
	
		response = self.send_request(req)
		if response is None:
			return
		services.ECUReset.interpret_response(response)
		
		if response.service_data.reset_type_echo != reset_type:
			raise UnexpectedResponseException(response, "Response subfunction received from server (0x%02x) does not match the requested subfunction (0x%02x)" % (response.service_data.reset_type_echo, reset_type))

		if response.service_data.reset_type_echo == services.ECUReset.ResetType.enableRapidPowerShutDown and response.service_data.powerdown_time != 0xFF:
			self.logger.info('Server will shutdown in %d seconds.' % (response.service_data.powerdown_time))

		return response

	@standard_error_management 
	def clear_dtc(self, group=0xFFFFFF):
		"""
		Requests the server to clear its active Diagnostic Trouble Codes with the :ref:`ClearDiagnosticInformation<ClearDiagnosticInformation>` service.

		:dependent configuration: ``exception_on_<type>_response``

		:param group: The group of DTCs to clear. It may refer to Powertrain DTCs, Chassis DTCs, etc. Values are defined by the ECU manufacturer except for two specific values

			- ``0x000000`` : Emissions-related systems
			- ``0xFFFFFF`` : All DTCs
		:type group: int

		:return: The server response parsed by :meth:`ClearDiagnosticInformation.interpret_response<udsoncan.services.ClearDiagnosticInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""

		request = services.ClearDiagnosticInformation.make_request(group)
		if group == 0xFFFFFF:
			self.logger.info('Clearing all DTCs (group mask : 0xFFFFFF)')
		else:
			self.logger.info('Clearing DTCs matching group mask : 0x%06x' % group)

		response = self.send_request(request)
		if response is None:
			return
		services.ClearDiagnosticInformation.interpret_response(response)

		return response

	# Performs a RoutineControl Service request
	def start_routine(self, routine_id, data=None):
		"""
		Requests the server to start a routine through the :ref:`RoutineControl<RoutineControl>` service (subfunction = 0x01).

		:dependent configuration: ``exception_on_<type>_response``

		:param routine_id: The 16-bit numerical ID of the routine to start
		:type group: int

		:param data: Optional additional data to give to the server
		:type data: bytes

		:return: The server response parsed by :meth:`RoutineControl.interpret_response<udsoncan.services.RoutineControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.routine_control(routine_id, services.RoutineControl.ControlType.startRoutine, data)

	# Performs a RoutineControl Service request
	def stop_routine(self, routine_id, data=None):
		"""
		Requests the server to stop a routine through the :ref:`RoutineControl<RoutineControl>` service (subfunction = 0x02).

		:dependent configuration: ``exception_on_<type>_response``

		:param routine_id: The 16-bit numerical ID of the routine to stop
		:type group: int

		:param data: Optional additional data to give to the server
		:type data: bytes

		:return: The server response parsed by :meth:`RoutineControl.interpret_response<udsoncan.services.RoutineControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.routine_control(routine_id, services.RoutineControl.ControlType.stopRoutine, data)

	# Performs a RoutineControl Service request
	def get_routine_result(self, routine_id, data=None):
		"""
		Requests the server to send back the execution result of the specified routine through the :ref:`RoutineControl<RoutineControl>` service (subfunction = 0x03).

		:dependent configuration: ``exception_on_<type>_response``

		:param routine_id: The 16-bit numerical ID of the routine
		:type group: int

		:param data: Optional additional data to give to the server
		:type data: bytes

		:return: The server response parsed by :meth:`RoutineControl.interpret_response<udsoncan.services.RoutineControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.routine_control(routine_id, services.RoutineControl.ControlType.requestRoutineResults, data)

	# Performs a RoutineControl Service request
	@standard_error_management
	def routine_control(self, routine_id, control_type, data=None):
		"""
		Sends a generic request for the :ref:`RoutineControl<RoutineControl>` service with custom subfunction (control_type).

		:dependent configuration: ``exception_on_<type>_response``

		:param control_type: The service subfunction. See :class:`RoutineControl.ControlType<udsoncan.services.RoutineControl.ControlType>`
		:type group: int

		:param routine_id: The 16-bit numerical ID of the routine
		:type group: int

		:param data: Optional additional data to give to the server
		:type data: bytes

		:return: The server response parsed by :meth:`RoutineControl.interpret_response<udsoncan.services.RoutineControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		request = services.RoutineControl.make_request(routine_id, control_type, data=data)
		payload_length = 0 if data is None else len(data)

		self.logger.info("Sending RoutineControl request with control type %s (0x%02x) for Routine ID : 0x%04x (%s) with a payload of %d bytes" % ( services.RoutineControl.ControlType.get_name(control_type), control_type, routine_id, Routine.name_from_id(routine_id), payload_length))

		response = self.send_request(request)
		if response is None:
			return
		services.RoutineControl.interpret_response(response)

		if control_type != response.service_data.control_type_echo:
			raise UnexpectedResponseException(response, "Control type of response (0x%02x) does not match request control type (0x%02x)" % (response.service_data.control_type_echo, control_type))

		if routine_id != response.service_data.routine_id_echo:
			raise UnexpectedResponseException(response, "Response received from server (ID = 0x%02x) is not for the requested routine ID (0x%02x)" % (response.service_data.routine_id_echo, routine_id))

		return response

	def read_extended_timing_parameters(self):
		"""
		Reads the timing parameters from the server with :ref:`AccessTimingParameter<AccessTimingParameter>` service with subfunction ``readExtendedTimingParameterSet`` (0x01).

		:dependent configuration: ``exception_on_<type>_response``
		
		:return: The server response parsed by :meth:`AccessTimingParameter.interpret_response<udsoncan.services.AccessTimingParameter.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.access_timing_parameter(access_type=services.AccessTimingParameter.AccessType.readExtendedTimingParameterSet)

	def reset_default_timing_parameters(self):
		"""
		Resets the server timing parameters to their default value with :ref:`AccessTimingParameter<AccessTimingParameter>` service with subfunction ``setTimingParametersToDefaultValues`` (0x02).

		:dependent configuration: ``exception_on_<type>_response``
		
		:return: The server response parsed by :meth:`AccessTimingParameter.interpret_response<udsoncan.services.AccessTimingParameter.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.access_timing_parameter(access_type=services.AccessTimingParameter.AccessType.setTimingParametersToDefaultValues)

	def read_active_timing_parameters(self):
		"""
		Reads the currently active timing parameters from the server with :ref:`AccessTimingParameter<AccessTimingParameter>` service with subfunction ``readCurrentlyActiveTimingParameters`` (0x03).

		:dependent configuration: ``exception_on_<type>_response``
		
		:return: The server response parsed by :meth:`AccessTimingParameter.interpret_response<udsoncan.services.AccessTimingParameter.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.access_timing_parameter(access_type=services.AccessTimingParameter.AccessType.readCurrentlyActiveTimingParameters)
	
	def set_timing_parameters(self, params):
		"""
		Sets the timing parameters into the server with :ref:`AccessTimingParameter<AccessTimingParameter>` service with subfunction ``setTimingParametersToGivenValues`` (0x04).

		:dependent configuration: ``exception_on_<type>_response``
		
		:param params: The parameters data. Specific to each ECU.
		:type params: bytes

		:return: The server response parsed by :meth:`AccessTimingParameter.interpret_response<udsoncan.services.AccessTimingParameter.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""
		return self.access_timing_parameter(access_type=services.AccessTimingParameter.AccessType.setTimingParametersToGivenValues, request_record=params)

	
	@standard_error_management
	def access_timing_parameter(self, access_type, timing_param_record=None):
		"""
		Sends a generic request for :ref:`AccessTimingParameter<AccessTimingParameter>` service with configurable subfunction (access_type).

		:dependent configuration: ``exception_on_<type>_response``

		:param access_type: The service subfunction. See :class:`AccessTimingParameter.AccessType<udsoncan.services.AccessTimingParameter.AccessType>`
		:type access_type: bytes

		:param params: The parameters data. Specific to each ECU.
		:type params: bytes

		:return: The server response parsed by :meth:`AccessTimingParameter.interpret_response<udsoncan.services.AccessTimingParameter.interpret_response>`
		:rtype: :ref:`Response<Response>`
	
		"""

		request = services.AccessTimingParameter.make_request(access_type, timing_param_record)
		payload_length = 0 if timing_param_record is None else len(timing_param_record)

		self.logger.info('Sending AccessTimingParameter service request with access type 0x%02x (%s) and a request record payload of %d bytes' % (access_type, services.AccessTimingParameter.AccessType.get_name(access_type), payload_length))

		response = self.send_request(request)
		if response is None:
			return

		services.AccessTimingParameter.interpret_response(response)

		if access_type != response.service_data.access_type_echo:
			raise UnexpectedResponseException(response, "Access type of response (0x%02x) does not match request access type (0x%02x)" % (response.service_data.access_type_echo, access_type))

		allowed_response_record_access_type = [
			services.AccessTimingParameter.AccessType.readExtendedTimingParameterSet, 
			services.AccessTimingParameter.AccessType.readCurrentlyActiveTimingParameters
		]

		if len(response.service_data.timing_param_record) > 0 and access_type not in allowed_response_record_access_type:
			self.logger.warning("Server returned data in the AccessTimingParameter response although none was asked")

		return response

	@standard_error_management
	def communication_control(self, control_type, communication_type):
		"""
		Switches the transmission or reception of certain messages on/off with :ref:`CommunicationControl<CommunicationControl>` service.

		:dependent configuration: ``exception_on_<type>_response``
		
		:param control_type: The action to request such as enabling or disabling some messages. See :class:`CommunicationControl.ControlType<udsoncan.services.CommunicationControl.ControlType>`. This value can also be ECU manufacturer-specific
		:type control_type: int

		:param communication_type: Indicates what section of the network and the type of message that should be affected by the command. Refer to :ref:`CommunicationType<CommunicationType>` for more details. If an `integer` or a `bytes` is given, the value will be decoded to create the required :ref:`CommunicationType<CommunicationType>` object
		:type communication_type: :ref:`CommunicationType<CommunicationType>`, bytes, int

		:return: The server response parsed by :meth:`CommunicationControl.interpret_response<udsoncan.services.CommunicationControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""

		communication_type = services.CommunicationControl.normalize_communication_type(communication_type)

		request = services.CommunicationControl.make_request(control_type, communication_type)
		self.logger.info('Sending CommunicationControl service request with control type 0x%02x (%s) and a communication type byte of 0x%02x (%s)' % (control_type, services.CommunicationControl.ControlType.get_name(control_type), communication_type.get_byte_as_int(), str(communication_type)))

		response = self.send_request(request)
		if response is None:
			return

		services.CommunicationControl.interpret_response(response)

		if control_type != response.service_data.control_type_echo:
			raise UnexpectedResponseException(response, "Control type of response (0x%02x) does not match request control type (0x%02x)" % (response.service_data.control_type_echo, control_type))

		return response

	def request_download(self, memory_location, dfi=None):
		"""
		Informs the server that the client wants to initiate a download from the client to the server by sending a :ref:`RequestDownload<RequestDownload>` service request.

		:dependent configuration: ``exception_on_<type>_response`` ``server_address_format`` ``server_memorysize_format``

		:param memory_location: The address and size of the memory block to be written.
		:type memory_location: :ref:`MemoryLocation <MemoryLocation>`

		:param dfi: Optional :ref:`DataFormatIdentifier <DataFormatIdentifier>` defining the compression and encryption scheme of the data. 
			If not specified, the default value of 00 will be used, specifying no encryption and no compression
		:type dfi: :ref:`DataFormatIdentifier <DataFormatIdentifier>`

		:return: The server response parsed by :meth:`RequestDownload.interpret_response<udsoncan.services.RequestDownload.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.request_upload_download(services.RequestDownload, memory_location, dfi)

	def request_upload(self, memory_location, dfi=None):
		"""
		Informs the server that the client wants to initiate an upload from the server to the client by sending a :ref:`RequestUpload<RequestUpload>` service request.

		:dependent configuration: ``exception_on_<type>_response`` ``server_address_format`` ``server_memorysize_format``
		
		:param memory_location: The address and size of the memory block to be written.
		:type memory_location: :ref:`MemoryLocation <MemoryLocation>`

		:param dfi: Optional :ref:`DataFormatIdentifier <DataFormatIdentifier>` defining the compression and encryption scheme of the data. 
			If not specified, the default value of 00 will be used, specifying no encryption and no compression
		:type dfi: :ref:`DataFormatIdentifier <DataFormatIdentifier>`

		:return: The server response parsed by :meth:`RequestUpload.interpret_response<udsoncan.services.RequestUpload.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.request_upload_download(services.RequestUpload, memory_location, dfi)

	# Common code for both RequestDownload and RequestUpload services
	@standard_error_management
	def request_upload_download(self, service_cls, memory_location, dfi=None):
		dfi = service_cls.normalize_data_format_identifier(dfi)

		if service_cls not in [services.RequestDownload, services.RequestUpload]:
			raise ValueError('services must either be RequestDownload or RequestUpload')

		if not isinstance(memory_location, MemoryLocation):
			raise ValueError('memory_location must be an instance of MemoryLocation')

		# If user does not specify a byte format, we apply the one in client configuration.
		if 'server_address_format' in self.config:
			memory_location.set_format_if_none(address_format=self.config['server_address_format'])

		if 'server_memorysize_format' in self.config:
			memory_location.set_format_if_none(memorysize_format=self.config['server_memorysize_format'])

		request = service_cls.make_request(memory_location=memory_location, dfi=dfi)
		self.logger.info('Sending %s service request for memory location [%s] and DataFormatIdentifier 0x%02x (%s)' % (service_cls.__name__, str(memory_location), dfi.get_byte_as_int(), str(dfi)))

		response = self.send_request(request)
		if response is None:
			return
		service_cls.interpret_response(response)
		
		return response

	@standard_error_management
	def transfer_data(self, sequence_number, data=None):
		"""
		Transfer a block of data to/from the client to/from the server by sending a :ref:`TransferData<TransferData>` service request and returning the server response.

		:dependent configuration: ``exception_on_<type>_response``
		
		:param sequence_number: Corresponds to an 8bit counter that should increment for each new block transferred.
			Allowed values are from 0 to 0xFF
		:type sequence_number: int

		:param data: Optional additional data to send to the server
		:type data: bytes

		:return: The server response parsed by :meth:`TransferData.interpret_response<udsoncan.services.TransferData.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		request = services.TransferData.make_request(sequence_number, data)
		
		data_len = 0 if data is None else len(data)
		self.logger.info('Sending TransferData service request with SequenceNumber=%d and %d bytes of data.' % (sequence_number, data_len))
		if data_len > 0:
			self.logger.debug('Data to transfer : %s' % binascii.hexlify(data))
		
		response = self.send_request(request)
		if response is None:
			return
		services.TransferData.interpret_response(response)

		if sequence_number != response.service_data.sequence_number_echo:
			raise UnexpectedResponseException(response, "Block sequence number of response (0x%02x) does not match request block sequence number (0x%02x)" % (response.service_data.sequence_number_echo, sequence_number))

		return response

	@standard_error_management
	def request_transfer_exit(self, data=None):
		"""
		Informs the server that the client wants to stop the data transfer by sending a :ref:`RequestTransferExit<RequestTransferExit>` service request.

		:dependent configuration: ``exception_on_<type>_response``

		:param data: Optional additional data to send to the server
		:type data: bytes

		:return: The server response parsed by :meth:`RequestTransferExit.interpret_response<udsoncan.services.RequestTransferExit.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		request = services.RequestTransferExit.make_request(data)
		self.logger.info('Sending RequestTransferExit service request')

		response = self.send_request(request)
		if response is None:
			return
		services.RequestTransferExit.interpret_response(response)

		return response

	@standard_error_management
	def link_control(self, control_type, baudrate=None):
		"""
		Controls the communication baudrate by sending a :ref:`LinkControl<LinkControl>` service request.

		:dependent configuration: ``exception_on_<type>_response``

		:param control_type: Allowed values are from 0 to 0xFF. See :class:`LinkControl.ControlType<udsoncan.services.LinkControl.ControlType>`
		:type control_type: int

		:param baudrate: Required baudrate value when ``control_type`` is either ``verifyBaudrateTransitionWithFixedBaudrate`` (1) or ``verifyBaudrateTransitionWithSpecificBaudrate`` (2)
		:type baudrate: :ref:`Baudrate <Baudrate>`

		:return: The server response parsed by :meth:`LinkControl.interpret_response<udsoncan.services.LinkControl.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		request = services.LinkControl.make_request(control_type, baudrate)
		baudrate_str = 'No baudrate specified' if baudrate is None else 'Baudrate : ' + str(baudrate)
		self.logger.info('Sending LinkControl service request with control type of 0x%02x (%s). %s' % (control_type, services.LinkControl.ControlType.get_name(control_type), baudrate_str))
		
		response = self.send_request(request)
		if response is None:
			return
		services.LinkControl.interpret_response(response)
		
		if control_type != response.service_data.control_type_echo:
			raise UnexpectedResponseException(response, "Control type of response (0x%02x) does not match request control type (0x%02x)" % (response.service_data.control_type_echo, control_type))

		return response

	@standard_error_management
	def io_control(self, did, control_param=None, values=None, masks=None):
		"""
		Substitutes the value of an input signal or overrides the state of an output by sending a :ref:`InputOutputControlByIdentifier<InputOutputControlByIdentifier>` service request.

		:dependent configuration: ``exception_on_<type>_response`` ``input_output`` ``tolerate_zero_padding``

		:param did: Data identifier to represent the IO
		:type did: int

		:param control_param: Optional parameter that can be a value from :class:`InputOutputControlByIdentifier.ControlParam<udsoncan.services.InputOutputControlByIdentifier.ControlParam>`
		:type control_param: int

		:param values: Optional values to send to the server. This parameter will be given to :ref:`DidCodec<DidCodec>`.encode() method. 
			It can be:
			
				- A list for positional arguments
				- A dict for named arguments
				- An instance of :ref:`IOValues<IOValues>` for mixed arguments

		:type values: list, dict, :ref:`IOValues<IOValues>`

		:param masks: Optional mask record for composite values. The mask definition must be included in ``config['input_output']``
			It can be:

				- A list naming the bit mask to set
				- A dict with the mask name as a key and a boolean setting or clearing the mask as the value
				- An instance of :ref:`IOMask<IOMask>`
				- A boolean value to set all masks to the same value.
		:type masks: list, dict, :ref:`IOMask<IOMask>`, bool

		:return: The server response parsed by :meth:`InputOutputControlByIdentifier.interpret_response<udsoncan.services.InputOutputControlByIdentifier.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		if 'input_output' not in self.config:
			raise ConfigError('input_output', msg='input_output must be defined in client configuration in order to use InputOutputControlByIdentifier service')

		request = services.InputOutputControlByIdentifier.make_request( did, control_param=control_param, values=values, masks=masks, ioconfig=self.config['input_output'])
		
		control_param_str = 'no control parameter' if control_param is None else 'control parameter 0x%02x (%s)' % (control_param, services.InputOutputControlByIdentifier.ControlParam.get_name(control_param))		
		self.logger.info('Sending InputOutputControlByIdentifier service request for DID=0x%04x, %s.' % (did, control_param_str))

		response = self.send_request(request)
		if response is None:
			return
		services.InputOutputControlByIdentifier.interpret_response(response, control_param=control_param, tolerate_zero_padding=self.config['tolerate_zero_padding'], ioconfig=self.config['input_output'])

		if response.service_data.did_echo != did:
			raise UnexpectedResponseException(response, "Echo of the DID number (0x%02x) does not match the value in the request (0x%02x)" % (response.service_data.did_echo, did))

		if control_param != response.service_data.control_param_echo:
			raise UnexpectedResponseException(response, 'Echo of the InputOutputControlParameter (0x%02x) does not match the value in the request (0x%02x)' % (response.service_data.control_param_echo, control_param))	

		return response

	@standard_error_management
	def control_dtc_setting(self, setting_type, data=None):
		"""
		Controls some settings related to the Diagnostic Trouble Codes by sending a :ref:`ControlDTCSetting<ControlDTCSetting>` service request. 
		It can enable/disable some DTCs or perform some ECU specific configuration.

		:dependent configuration: ``exception_on_<type>_response``

		:param setting_type: Allowed values are from 0 to 0x7F. See :class:`ControlDTCSetting.SettingType<udsoncan.services.ControlDTCSetting.SettingType>`
		:type setting_type: int

		:param data: Optional additional data sent with the request called `DTCSettingControlOptionRecord`
		:type data: bytes

		:return: The server response parsed by :meth:`ControlDTCSetting.interpret_response<udsoncan.services.ControlDTCSetting.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		request = services.ControlDTCSetting.make_request(setting_type, data)
		data_len = 0 if data is None else len(data)
		self.logger.info('Sending ControlDTCSetting service request with setting type 0x%02x (%s) and %d bytes of data' % (setting_type, services.ControlDTCSetting.SettingType.get_name(setting_type), data_len))

		response = self.send_request(request)
		if response is None:
			return
		services.ControlDTCSetting.interpret_response(response)

		if response.service_data.setting_type_echo != setting_type:
			raise UnexpectedResponseException(response, "Setting type of response (0x%02x) does not match request control type (0x%02x)" % (response.service_data.setting_type_echo, setting_type))

		return response

	@standard_error_management
	def read_memory_by_address(self, memory_location):
		"""
		Reads a block of memory from the server by sending a :ref:`ReadMemoryByAddress<ReadMemoryByAddress>` service request. 

		:dependent configuration: ``exception_on_<type>_response`` ``server_address_format`` ``server_memorysize_format``

		:param memory_location: The address and the size of the memory block to read.
		:type memory_location: :ref:`MemoryLocation <MemoryLocation>`

		:return: The server response parsed by :meth:`ReadMemoryByAddress.interpret_response<udsoncan.services.ReadMemoryByAddress.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		if not isinstance(memory_location, MemoryLocation):
			raise ValueError('memory_location must be an instance of MemoryLocation')

		if 'server_address_format' in self.config:
			memory_location.set_format_if_none(address_format=self.config['server_address_format'])

		if 'server_memorysize_format' in self.config:
			memory_location.set_format_if_none(memorysize_format=self.config['server_memorysize_format'])

		request = services.ReadMemoryByAddress.make_request(memory_location)
		self.logger.info('Reading memory address (ReadMemoryByAddress) - %s' % str(memory_location))
		
		response = self.send_request(request)
		if response is None:
			return
		services.ReadMemoryByAddress.interpret_response(response)
		memdata = response.service_data.memory_block
		
		if len(memdata) < memory_location.memorysize:
			raise UnexpectedResponseException(response, 'Data block given by the server is too short. Client requested for %d bytes but only received %d bytes' % (memory_location.memorysize, len(response.data)))

		if len(memdata) > memory_location.memorysize:
			extra_bytes = len(memdata) - memory_location.memorysize
			if memdata[memory_location.memorysize:] == b'\x00' * extra_bytes and self.config['tolerate_zero_padding']:
				response.service_data.memory_block = memdata[0:memory_location.memorysize]	# trim exceeding zeros
			else:
				raise UnexpectedResponseException(response, 'Data block given by the server is too long. Client requested for %d bytes but received %d bytes' % (memory_location.memorysize, len(response.data)))

		return response

	@standard_error_management
	def write_memory_by_address(self, memory_location, data):
		"""
		Writes a block of memory in the server by sending a :ref:`WriteMemoryByAddress<WriteMemoryByAddress>` service request. 

		:dependent configuration: ``exception_on_<type>_response`` ``server_address_format`` ``server_memorysize_format``

		:param memory_location: The address and the size of the memory block to read. 
		:type memory_location: :ref:`MemoryLocation <MemoryLocation>`

		:param data: The data to write into memory.
		:type data: bytes

		:return: The server response parsed by :meth:`WriteMemoryByAddress.interpret_response<udsoncan.services.WriteMemoryByAddress.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""

		if not isinstance(memory_location, MemoryLocation):
			raise ValueError('memory_location must be an instance of MemoryLocation')

		if 'server_address_format' in self.config:
			memory_location.set_format_if_none(address_format=self.config['server_address_format'])

		if 'server_memorysize_format' in self.config:
			memory_location.set_format_if_none(memorysize_format=self.config['server_memorysize_format'])

		request = services.WriteMemoryByAddress.make_request(memory_location, data)
		self.logger.info('Writing %d bytes to memory address (WriteMemoryByAddress) - %s' % (len(data), str(memory_location)))

		if len(data) != memory_location.memorysize:
			self.logger.warning('WriteMemoryByAddress: data block length (%d bytes) does not match MemoryLocation size (%d bytes)' % (len(data, memory_location.memorysize)))

		response = self.send_request(request)
		if response is None:
			return
		services.WriteMemoryByAddress.interpret_response(response, memory_location)

		alfid_byte 			= memory_location.alfid.get_byte_as_int()   # AddressAndLengthFormatIdentifier
		address_bytes 		= memory_location.get_address_bytes()
		memorysize_bytes 	= memory_location.get_memorysize_bytes()
		
		# We make sure that the echo from the server matches the request we sent.
		if response.service_data.alfid_echo != alfid_byte:
			raise UnexpectedResponseException(response, 'AddressAndLengthFormatIdentifier echoed back by the server (0x%02X) does not match the one requested by the client (0x%02X)' % (response.service_data.alfid_echo, int(alfid_byte)) )
		
		if response.service_data.memory_location_echo.address != memory_location.address:
			raise UnexpectedResponseException(response, 'Address echoed back by the server (0x%X) does not match the one requested by the client (0x%X)' % (response.service_data.memory_location_echo.address, memory_location.address) )

		if response.service_data.memory_location_echo.memorysize != memory_location.memorysize:
			raise UnexpectedResponseException(response, 'Memory size echoed back by the server (0x%X) does not match the one requested by the client (0x%X)' % (response.service_data.memory_location_echo.memorysize, memory_location.memorysize) )

		return response

# ====  ReadDTCInformation
	def get_dtc_by_status_mask(self, status_mask):
		"""
		Reads all the Diagnostic Trouble Codes that have a status matching the given mask. 
		The server will check all of its DTCs and if (Dtc.status & status_mask) != 0, then the DTCs match the filter and are sent back to the client.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCByStatusMask, status_mask=status_mask)

	def get_emission_dtc_by_status_mask(self, status_mask):
		"""
		Reads the emission-related Diagnostic Trouble Codes that have a status matching the given mask.
		The server will check its emission-related DTCs and if (Dtc.status & status_mask) != 0, then the DTCs match the filter and are sent back to the client.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportEmissionsRelatedOBDDTCByStatusMask, status_mask=status_mask)

	def get_mirrormemory_dtc_by_status_mask(self, status_mask):
		"""
		Reads all the Diagnostic Trouble Codes stored in mirror memory that have a status matching the given mask. 
		The server will check all of its DTCs and if (Dtc.status & status_mask) != 0, then the DTCs match the filter and are sent back to the client.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportMirrorMemoryDTCByStatusMask, status_mask=status_mask)

	def get_dtc_by_status_severity_mask(self, status_mask, severity_mask):
		"""
		Reads all the Diagnostic Trouble Codes that have a status and a severity matching the given masks. 
		The server will check all of its DTCs and if ( (Dtc.status & status_mask) != 0 && (Dtc.severity & severity) !=0), then the DTCs match the filter and are sent back to the client.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`
		
		:param severity_mask: The severity mask against which the DTCs are tested. 
		:type severity_mask: int or :ref:`Dtc.Severity<DTC_Severity>`
		
		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCBySeverityMaskRecord, status_mask=status_mask, severity_mask=severity_mask)

	def get_number_of_dtc_by_status_mask(self, status_mask):
		"""
		Gets the number of DTCs that match the specified status mask.

		:dependent configuration: ``exception_on_<type>_response``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportNumberOfDTCByStatusMask, status_mask=status_mask)
	
	def get_mirrormemory_number_of_dtc_by_status_mask(self, status_mask):
		"""
		Gets the number of DTCs that match the specified status mask in mirror memory.

		:dependent configuration: ``exception_on_<type>_response``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportNumberOfMirrorMemoryDTCByStatusMask, status_mask=status_mask)
	
	def get_number_of_emission_dtc_by_status_mask(self, status_mask):
		"""
		Gets the number of emission-related DTCs that match the specified status mask.

		:dependent configuration: ``exception_on_<type>_response``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportNumberOfEmissionsRelatedOBDDTCByStatusMask, status_mask=status_mask)

	def get_number_of_dtc_by_status_severity_mask(self, status_mask, severity_mask):
		"""
		Gets the number of DTCs that match the specified status mask and severity mask.

		:dependent configuration: ``exception_on_<type>_response``

		:param status_mask: The status mask against which the DTCs are tested. 
		:type status_mask: int or :ref:`Dtc.Status<DTC_Status>`

		:param severity_mask: The severity mask against which the DTCs are tested. 
		:type severity_mask: int or :ref:`Dtc.Severity<DTC_Severity>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportNumberOfDTCBySeverityMaskRecord, status_mask=status_mask, severity_mask=severity_mask)
	
	def get_dtc_severity(self, dtc):
		"""
		Requests the server for a specific DTC severity level.

		:dependent configuration: ``exception_on_<type>_response``

		:param dtc: The DTC ID for which we request the severity. It can be a 3-byte integer or a DTC instance with an ID set.
		:type dtc: int or :ref:`Dtc<DTC>`

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportSeverityInformationOfDTC, dtc=dtc)

	def get_supported_dtc(self):
		"""
		Requests the list of supported DTCs by the server.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportSupportedDTCs)

	def get_first_test_failed_dtc(self):
		"""
		Reads a single DTC. Requests the server for the first DTC that set its ``Dtc.Status.test_failed`` bit.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportFirstTestFailedDTC)

	def get_first_confirmed_dtc(self):
		"""
		Reads a single DTC. Requests the server for the first DTC that set its ``Dtc.Status.confirmed`` bit.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportFirstConfirmedDTC)

	def get_most_recent_test_failed_dtc(self):
		"""
		Reads a single DTC. Requests the server for the last DTC that set its ``Dtc.Status.test_failed`` bit.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportMostRecentTestFailedDTC)

	def get_most_recent_confirmed_dtc(self):
		"""
		Reads a single DTC. Requests the server for the last DTC that set its ``Dtc.Status.confirmed`` bit.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportMostRecentConfirmedDTC)

	def get_dtc_with_permanent_status(self):
		"""
		Returns all DTCs that the server marked as `permanent`. 

		A permanent DTC is a DTC stored in Non-Volatile memory and that cannot be erased by test equipment or by power-cycling the ECU.
		
		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCWithPermanentStatus)

	def get_dtc_fault_counter(self):
		"""
		Requests the server for all DTCs that are `prefailed` along with their fault detection counter. 

		A prefailed DTC is a DTC for which the detection condition is met, but has not been identified as `pending` or `confirmed` yet. 

		If the ECU follows the UDS guidelines, it will wait to detect a fault many times before setting a status bit for this fault DTC. Each time the fault is detected, a fault counter is incremented, when it is not detected, the counter is decremented.
		Once the fault counter reaches a threshold, a status bit is set and the DTC is not `prefailed` anymore. A `prefailed` DTC is any DTC that has fault detection counter greater than 0, but less than the detection threshold.
		
		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCFaultDetectionCounter)

	def get_dtc_snapshot_identification(self):
		"""
		Requests the server to return an index of all the DTC snapshots available. The server will respond with a list of DTCs and a list of snapshot record numbers for each DTC.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc``

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCSnapshotIdentification)

	def get_dtc_snapshot_by_dtc_number(self, dtc, record_number=0xFF):
		"""
		Requests the server for one or many specific DTC snapshots associated with a single DTC.
		Each snapshot has a data identifier associated with it. The data will be decoded using the associated :ref:`DidCodec<DidCodec>` defined in ``config['data_identifiers']``.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``ignore_all_zero_dtc`` ``dtc_snapshot_did_size``

		:param dtc: The DTC ID for which we request the snapshot data. It can be a 3-byte integer or a DTC instance with an ID set.
		:type dtc: int or :ref:`Dtc<DTC>`

		:param record_number: The record number of the snapshot data to read. If 0xFF is given, then all snapshots will be read, otherwise, a single snapshot will be read.
		:type record_number: int

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByDTCNumber, dtc=dtc, snapshot_record_number=record_number)

	def get_dtc_snapshot_by_record_number(self, record_number=0xFF):
		"""
		Requests the server for one or many DTC snapshots by specifying a record number. This functionality can exist only if the server assigns globally unique record_numbers to DTC snapshots, regardless of the DTC ID.

		Each snapshot has a data identifier associated with it. The data will be decoded using the associated :ref:`DidCodec<DidCodec>` defined in ``config['data_identifiers']``.

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding``  ``dtc_snapshot_did_size``

		:param record_number: The record number of the snapshot data to read. If 0xFF is given, then all snapshots will be read, otherwise, a single snapshot will be read.
		:type record_number: int

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByRecordNumber, snapshot_record_number=record_number)

	def get_dtc_extended_data_by_dtc_number(self, dtc, record_number=0xFF, data_size = None):
		"""
		Requests the server for one or many DTC **extended data** by specifying a record number.

		The DTC extended data is an ECU specific set of data that is not associated with a data identifier. Given as ``bytes``

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``extended_data_size``

		:param dtc: The DTC ID for which we request the extended data. It can be a 3-byte integer or a DTC instance with an ID set.
		:type dtc: int or :ref:`Dtc<DTC>`

		:param record_number: The record number of the extended data to read. If 0xFF is given, then all extended data entries will be read, otherwise, a single entry will be read.
		:type record_number: int

		:param data_size: The number of bytes of an extended data record. If not specified ``config['extended_data_size'][dtc]`` will be used.
		:type data_size: int or None

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportDTCExtendedDataRecordByDTCNumber, dtc=dtc, extended_data_record_number=record_number,extended_data_size=data_size)

	def get_mirrormemory_dtc_extended_data_by_dtc_number(self, dtc, record_number=0xFF, data_size = None):
		"""
		Requests the server for one or many DTC **extended data** stored in mirror memory by specifying a record number.

		The DTC extended data is an ECU specific set of data that is not associated with a data identifier. Given as ``bytes``

		:dependent configuration: ``exception_on_<type>_response`` ``tolerate_zero_padding`` ``extended_data_size``

		:param dtc: The DTC ID for which we request the extended data. It can be a 3-byte integer or a DTC instance with an ID set.
		:type dtc: int or :ref:`Dtc<DTC>`

		:param record_number: The record number of the extended data to read. If 0xFF is given, then all extended data entries will be read, otherwise, a single entry will be read.
		:type record_number: int

		:param data_size: The number of bytes of an extended data record. If not specified ``config['extended_data_size'][dtc]`` wil be used.
		:type data_size: int or None

		:return: The server response parsed by :meth:`ReadDTCInformation.interpret_response<udsoncan.services.ReadDTCInformation.interpret_response>`
		:rtype: :ref:`Response<Response>`
		"""
		return self.read_dtc_information(services.ReadDTCInformation.Subfunction.reportMirrorMemoryDTCExtendedDataRecordByDTCNumber, dtc=dtc, extended_data_record_number=record_number, extended_data_size=data_size)

	# Performs a ReadDiagnsticInformation service request.
	# Many requests are encoded the same way and many responses are encoded the same way. Request grouping and response grouping are independent.
	@standard_error_management
	def read_dtc_information(self, subfunction, status_mask=None, severity_mask=None,  dtc=None, snapshot_record_number=None, extended_data_record_number=None, extended_data_size=None):
		if dtc is not None and isinstance(dtc, Dtc):
			dtc = dtc.id

		request_params = {
			'subfunction' : subfunction,
			'status_mask' : status_mask,
			'severity_mask' : severity_mask,
			'dtc' : dtc,
			'snapshot_record_number' : snapshot_record_number,
			'extended_data_record_number' : extended_data_record_number
		}

		request = services.ReadDTCInformation.make_request(**request_params)

		self.logger.info('Sending a ReadDtcInformation service request with subfunction "%s" (0x%02X).' % (services.ReadDTCInformation.Subfunction.get_name(subfunction), subfunction))
		self.logger.debug('Params are : %s' % str(request_params))
		response = self.send_request(request)
		if response is None:
			return

		response_params = {
			'subfunction' : subfunction,
			'tolerate_zero_padding' : self.config['tolerate_zero_padding'],
			'ignore_all_zero_dtc' : self.config['ignore_all_zero_dtc'],
			'dtc_snapshot_did_size' : self.config['dtc_snapshot_did_size'],
			'didconfig' : self.config['data_identifiers'] if 'data_identifiers' in self.config else None,
			'extended_data_size' : extended_data_size
		}

		if extended_data_size is None:
			if 'extended_data_size' in self.config:
				if dtc is not None and dtc in self.config['extended_data_size']:
					response_params['extended_data_size'] = self.config['extended_data_size'][dtc]

		services.ReadDTCInformation.interpret_response(response, **response_params)
		
		if response.service_data.subfunction_echo != subfunction:
			raise UnexpectedResponseException(response, 'Echo of ReadDTCInformation subfunction gotten from server(0x%02x) does not match the value in the request subfunction (0x%02x)' % (response.service_data.subfunction_echo, subfunction))	

		if subfunction == services.ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByDTCNumber:
			if len(response.service_data.dtcs) == 1:
				if dtc != response.service_data.dtcs[0].id:
					raise UnexpectedResponseException(response, 'Server returned snapshot with DTC ID 0x%06x while client requested for 0x%06x' % (response.service_data.dtcs[0].id, dtc))

		if subfunction in [services.ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByRecordNumber, services.ReadDTCInformation.Subfunction.reportDTCSnapshotRecordByDTCNumber]:
			if len(response.service_data.dtcs) == 1 and snapshot_record_number != 0xFF:
				for snapshot in response.service_data.dtcs[0].snapshots:
					if snapshot.record_number != snapshot_record_number:
						raise UnexpectedResponseException(response, 'Server returned snapshot with record number 0x%02x while client requested for 0x%02x' % (snapshot.record_number, snapshot_record_number)) 

		if subfunction in [services.ReadDTCInformation.Subfunction.reportDTCExtendedDataRecordByDTCNumber, services.ReadDTCInformation.Subfunction.reportMirrorMemoryDTCExtendedDataRecordByDTCNumber]:
			if len(response.service_data.dtcs) == 1 and extended_data_record_number < 0xF0: # Standard specifies that values between 0xF0 and 0xFF are for reporting groups (more than one record)
				for extended_data in response.service_data.dtcs[0].extended_data:
					if extended_data.record_number != extended_data_record_number :	
						raise UnexpectedResponseException(response, 'Extended data record number given by the server (0x%02x) does not match the record number requested by the client (0x%02x)' % (extended_data.record_number, extended_data_record_number))

		for dtc in response.service_data.dtcs:
			if dtc.fault_counter is not None and (dtc.fault_counter >= 0x7F or dtc.fault_counter < 0x01):
				self.logger.warning('Server returned a fault counter value of 0x%02x for DTC id 0x%06x while value should be between 0x01 and 0x7E.' % (dtc.fault_counter, dtc.id))

		if Dtc.Format.get_name(response.service_data.dtc_format) is None:
			self.logger.warning('Unknown DTC Format Identifier 0x%02x. Value should be between 0 and 3' % response.service_data.dtc_format)

		return response

	# Basic transmission of requests. This will need to be improved
	def send_request(self, request, timeout=-1):
		if timeout is not None and timeout < 0:
			timeout = self.request_timeout
		self.conn.empty_rxqueue()
		self.logger.debug("Sending request to server")
		override_suppress_positive_response = False
		if self.suppress_positive_response.enabled == True and request.service.use_subfunction():
			payload = request.get_payload(suppress_positive_response=True)
			override_suppress_positive_response = True
		else:
			payload = request.get_payload()

		if self.suppress_positive_response.enabled and not request.service.use_subfunction():
			self.logger.warning('SuppressPositiveResponse cannot be used for service %s. Ignoring' % (request.service.get_name()))

		self.conn.send(payload)

		if request.suppress_positive_response  or override_suppress_positive_response:
			return
		
		self.logger.debug("Waiting for server response")
		try:
			payload = self.conn.wait_frame(timeout=timeout, exception=True)
		except Exception as e:
			raise e

		response = Response.from_payload(payload)
		self.last_response = response
		self.logger.debug("Received response from server")

		if not response.valid:
			raise InvalidResponseException(response, 'Received response is invalid.')
				
		if response.service.response_id() != request.service.response_id():
			msg = "Response gotten from server has a service ID different than the one of the request. Received=0x%02x, Expected=0x%02x" % (response.service.response_id() , request.service.response_id() )
			raise UnexpectedResponseException(response, msg)
		
		if not response.positive:
			if not request.service.is_supported_negative_response(response.code):
				self.logger.warning("Given response (%s) is not a supported negative response code according to UDS standard." % response.code_name)	
			raise NegativeResponseException(response)

		self.logger.info('Received positive response from server.')
		return response



from udsoncan import DataFormatIdentifier, AddressAndLengthFormatIdentifier,MemoryLocation, CommunicationType, Baudrate, IOMasks, IOValues, Dtc, DidCodec, AsciiCodec
from test.UdsTest import UdsTest
import struct

class TestAddressAndLengthFormatIdentifier(UdsTest):
	def test_ali_1(self):
		alfid = AddressAndLengthFormatIdentifier(memorysize_format=8, address_format=8)
		self.assertEqual(alfid.get_byte(),b'\x11')

	def test_ali_2(self):
		alfid = AddressAndLengthFormatIdentifier(memorysize_format=16, address_format=8)
		self.assertEqual(alfid.get_byte(),b'\x21')

	def test_ali_oob_values(self):	# Out Of Bounds Value
		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=1, address_format=1)
		
		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=0, address_format=8)

		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=8, address_format=0)

		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=40, address_format=0)

		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=8, address_format=48)

		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format='8', address_format=8)

		with self.assertRaises(ValueError):
			AddressAndLengthFormatIdentifier(memorysize_format=8, address_format='8')

	def test_str_repr(self):
		alfid = AddressAndLengthFormatIdentifier(memorysize_format=8, address_format=8)
		str(alfid)
		alfid.__repr__()

class TestDataFormatIdentifier(UdsTest):
	def test_dfi(self):
		dfi = DataFormatIdentifier(compression=1, encryption=2)
		self.assertEqual(dfi.get_byte(),b'\x12')

	def test_dfi2(self):
		dfi = DataFormatIdentifier(compression=15, encryption=15)
		self.assertEqual(dfi.get_byte(),b'\xFF')

	def test_str_repr(self):
		dfi = DataFormatIdentifier(compression=1, encryption=2)
		str(dfi)
		dfi.__repr__()

	def test_ali_oob_values(self):
		with self.assertRaises(ValueError):
			DataFormatIdentifier(compression=-1, encryption=1)
		
		with self.assertRaises(ValueError):
			DataFormatIdentifier(compression=1, encryption=-1)

		with self.assertRaises(ValueError):
			DataFormatIdentifier(compression=16, encryption=1)

		with self.assertRaises(ValueError):
			DataFormatIdentifier(compression=1, encryption=16)

class TestMemoryLocation(UdsTest):
	def test_memloc1(self):
		memloc = MemoryLocation(address=0x1234, memorysize=0x78, address_format=16, memorysize_format=8)
		self.assertEqual(memloc.get_address_bytes(), b'\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x78')

	def test_memloc_autosize1(self):
		memloc = MemoryLocation(address=0x1234, memorysize=0x78)
		self.assertEqual(memloc.get_address_bytes(), b'\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x78')

	def test_memloc_autosize2(self):
		memloc = MemoryLocation(address=0x1234567, memorysize=0x789abb)
		self.assertEqual(memloc.get_address_bytes(), b'\x01\x23\x45\x67')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x78\x9a\xbb')

	def test_memloc_str_repr(self):
		memloc = MemoryLocation(address=0x1234, memorysize=0x78, address_format=16, memorysize_format=8)
		str(memloc)
		memloc.__repr__()

	def test_memloc_override(self):
		memloc = MemoryLocation(address=0x1234, memorysize=0x78)
		self.assertEqual(memloc.get_address_bytes(), b'\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x78')
		memloc.set_format_if_none(address_format=32)
		self.assertEqual(memloc.get_address_bytes(), b'\x00\x00\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x78')
		memloc.set_format_if_none(memorysize_format=24)
		self.assertEqual(memloc.get_address_bytes(), b'\x00\x00\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x00\x00\x78')

		memloc = MemoryLocation(address=0x1234, memorysize=0x78)
		memloc.set_format_if_none(address_format=32, memorysize_format=24)	# Both at same time.
		self.assertEqual(memloc.get_address_bytes(), b'\x00\x00\x12\x34')
		self.assertEqual(memloc.get_memorysize_bytes(), b'\x00\x00\x78')


	def test_memloc_from_bytes(self):
		memloc = MemoryLocation.from_bytes(address_bytes=b'\x12\x34', memorysize_bytes=b'\xFF')
		self.assertEqual(memloc.address, 0x1234)
		self.assertEqual(memloc.memorysize, 0xFF)
		self.assertEqual(memloc.address_format, 16)
		self.assertEqual(memloc.memorysize_format, 8)

		memloc = MemoryLocation.from_bytes(address_bytes=b'\x12\x34\x56', memorysize_bytes=b'\x66\x77\x88')
		self.assertEqual(memloc.address, 0x123456)
		self.assertEqual(memloc.memorysize, 0x667788)
		self.assertEqual(memloc.address_format, 24)
		self.assertEqual(memloc.memorysize_format, 24)

	def test_memloc_max_size(self):
		MemoryLocation.from_bytes(address_bytes=b'\x12\x34\x56\x78\x9a', memorysize_bytes=b'\xFF')
		with self.assertRaises(ValueError):
			MemoryLocation.from_bytes(address_bytes=b'\x12\x34\x56\x78\x9a\xbc', memorysize_bytes=b'\xFF')

		MemoryLocation.from_bytes(address_bytes=b'\x12\x34', memorysize_bytes=b'\x12\x34\x56\x78')
		with self.assertRaises(ValueError):
			MemoryLocation.from_bytes(address_bytes=b'\x12\x34', memorysize_bytes=b'\x12\x34\x56\x78\x9a')


class TestCommunicationType(UdsTest):
	def test_make(self):
		comtype = CommunicationType(subnet=CommunicationType.Subnet.node, normal_msg=True, network_management_msg=False)
		self.assertEqual(comtype.get_byte(), b'\x01')

		comtype = CommunicationType(subnet=CommunicationType.Subnet.network, normal_msg=True, network_management_msg=False)
		self.assertEqual(comtype.get_byte(), b'\xF1')

		comtype = CommunicationType(subnet=3, normal_msg=True, network_management_msg=True)
		self.assertEqual(comtype.get_byte(), b'\x33')

	def test_str_repr(self):
		comtype = CommunicationType(subnet=CommunicationType.Subnet.node, normal_msg=True, network_management_msg=False)
		str(comtype)
		comtype.__repr__()

	def test_from_byte(self):
		comtype = CommunicationType.from_byte(b'\x01')
		self.assertEqual(comtype.get_byte(), b'\x01')

		comtype = CommunicationType.from_byte(b'\xF1')
		self.assertEqual(comtype.get_byte(), b'\xF1')

		comtype = CommunicationType.from_byte(b'\x33')
		self.assertEqual(comtype.get_byte(), b'\x33')

		comtype = CommunicationType.from_byte(0x01)
		self.assertEqual(comtype.get_byte(), b'\x01')

	def test_oob_values(self):
		with self.assertRaises(ValueError):
			CommunicationType(subnet=0, normal_msg=False, network_management_msg=False)

		with self.assertRaises(ValueError):
			CommunicationType(subnet='x', normal_msg=True, network_management_msg=False)

		with self.assertRaises(ValueError):
			CommunicationType(subnet=0, normal_msg=1, network_management_msg=True)

		with self.assertRaises(ValueError):
			CommunicationType(subnet=0, normal_msg=True, network_management_msg=1)

class TestBaudrate(UdsTest):
	def test_create_fixed(self):
		br = Baudrate(115200, baudtype=Baudrate.Type.Fixed)
		self.assertEqual(br.get_bytes(), b'\x05')

		with self.assertRaises(ValueError):
			br = Baudrate(123456, baudtype=Baudrate.Type.Fixed)

	def test_create_specific(self):
		br = Baudrate(115200, baudtype=Baudrate.Type.Specific)
		self.assertEqual(br.get_bytes(), b'\x01\xC2\x00')

		with self.assertRaises(ValueError):
			br = Baudrate(0x1000000, baudtype=Baudrate.Type.Specific)

	def test_create_id(self):
		for i in range (0xFF):
			br = Baudrate(i, baudtype=Baudrate.Type.Identifier)
			self.assertEqual(br.get_bytes(), struct.pack('B', i))

		with self.assertRaises(ValueError):
			br = Baudrate(0x100, baudtype=Baudrate.Type.Identifier)

	def test_effective_baudrate(self):
		br = Baudrate(0x12, Baudrate.Type.Identifier) # 500kbits
		self.assertEqual(br.effective_baudrate(), 500000)

	def test_change_type(self):
		br = Baudrate(115200, baudtype=Baudrate.Type.Fixed)
		br2 = br.make_new_type(Baudrate.Type.Specific)
		self.assertEqual(br2.get_bytes(), b'\x01\xC2\x00')

		br = Baudrate(115200, baudtype=Baudrate.Type.Specific)
		br2 = br.make_new_type(Baudrate.Type.Fixed)
		self.assertEqual(br2.get_bytes(), b'\x05')

	def test_str_repr(self):
		br = Baudrate(115200, baudtype=Baudrate.Type.Fixed)
		str(br)
		br.__repr__()

	def test_create_auto(self):
		# Direct ID
		br = Baudrate(1)
		self.assertEqual(br.get_bytes(), b'\x01')
		
		br = Baudrate(0xFF)
		self.assertEqual(br.get_bytes(), b'\xFF')

		# Fixed baudrate
		br = Baudrate(115200)
		self.assertEqual(br.get_bytes(), b'\x05')

		br = Baudrate(500000)
		self.assertEqual(br.get_bytes(), b'\x12')
		
		#Specific Baudrate:
		br = Baudrate(0x123456)
		self.assertEqual(br.get_bytes(), b'\x12\x34\x56')

	def test_oob_values(self):
		with self.assertRaises(ValueError):
			br = Baudrate(-1)

		with self.assertRaises(ValueError):
			br = Baudrate(1, baudtype=-1)

		with self.assertRaises(ValueError):
			br = Baudrate(1, baudtype=0xFF)

class TestIOMasks(UdsTest):
	def test_oob_values(self):
		with self.assertRaises(ValueError):
			IOMasks(aaa='asd')

		with self.assertRaises(ValueError):
			IOMasks(1,2,3)

	def test_make_dict(self):
		m = IOMasks('aaa', 'bbb') # Correct syntax
		self.assertEqual(m.get_dict(), {'aaa' : True, 'bbb' : True})

		m = IOMasks('aaa', 'bbb', ccc=True, ddd=False) # Correct syntax
		self.assertEqual(m.get_dict(), {'aaa' : True, 'bbb' : True, 'ccc':True, 'ddd':False})

class TestDtc(UdsTest):
	def test_init(self):
		dtc = Dtc(0x1234)
		self.assertEqual(dtc.id, 0x1234 )
		self.assertEqual(dtc.status.get_byte(), b'\x00')
		self.assertEqual(dtc.status.get_byte_as_int(), 0x00)
		self.assertEqual(dtc.severity.get_byte(), b'\x00')
		self.assertEqual(dtc.severity.get_byte_as_int(), 0x00)

		self.assertEqual(dtc.status.test_failed, False)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, False)
		self.assertEqual(dtc.status.pending, False)
		self.assertEqual(dtc.status.confirmed, False)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, False)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		with self.assertRaises(TypeError):
			Dtc()

	def test_set_status_with_byte_no_error(self):
		dtc=Dtc(1)
		for i in range(255):
			dtc.status.set_byte(i)

	def test_status_behaviour(self):
		dtc=Dtc(1)

		self.assertEqual(dtc.status.get_byte(), b'\x00')
		dtc.status.test_failed=True
		self.assertEqual(dtc.status.get_byte(), b'\x01')
		dtc.status.test_failed_this_operation_cycle = True
		self.assertEqual(dtc.status.get_byte(), b'\x03')
		dtc.status.pending = True
		self.assertEqual(dtc.status.get_byte(), b'\x07')
		dtc.status.confirmed = True
		self.assertEqual(dtc.status.get_byte(), b'\x0F')
		dtc.status.test_not_completed_since_last_clear = True
		self.assertEqual(dtc.status.get_byte(), b'\x1F')
		dtc.status.test_failed_since_last_clear = True
		self.assertEqual(dtc.status.get_byte(), b'\x3F')
		dtc.status.test_not_completed_this_operation_cycle = True
		self.assertEqual(dtc.status.get_byte(), b'\x7F')
		dtc.status.warning_indicator_requested = True
		self.assertEqual(dtc.status.get_byte(), b'\xFF')
		
		dtc.status.set_byte(0x01)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, False)
		self.assertEqual(dtc.status.pending, False)
		self.assertEqual(dtc.status.confirmed, False)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, False)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x03)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, False)
		self.assertEqual(dtc.status.confirmed, False)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, False)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x07)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, False)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, False)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x0F)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, True)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, False)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x1F)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, True)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, True)
		self.assertEqual(dtc.status.test_failed_since_last_clear, False)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x3F)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, True)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, True)
		self.assertEqual(dtc.status.test_failed_since_last_clear, True)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, False)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0x7F)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, True)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, True)
		self.assertEqual(dtc.status.test_failed_since_last_clear, True)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, True)
		self.assertEqual(dtc.status.warning_indicator_requested, False)

		dtc.status.set_byte(0xFF)
		self.assertEqual(dtc.status.test_failed, True)
		self.assertEqual(dtc.status.test_failed_this_operation_cycle, True)
		self.assertEqual(dtc.status.pending, True)
		self.assertEqual(dtc.status.confirmed, True)
		self.assertEqual(dtc.status.test_not_completed_since_last_clear, True)
		self.assertEqual(dtc.status.test_failed_since_last_clear, True)
		self.assertEqual(dtc.status.test_not_completed_this_operation_cycle, True)
		self.assertEqual(dtc.status.warning_indicator_requested, True)

	def test_set_severity_with_byte_no_error(self):
		dtc=Dtc(1)
		for i in range(255):
			dtc.severity.set_byte(i)

	def test_str_repr(self):
		dtc=Dtc(0x123456)
		dtc.status.pending = True
		str(dtc)
		dtc.__repr__()


	def test_severity_behaviour(self):
		dtc=Dtc(1)

		self.assertEqual(dtc.severity.get_byte_as_int(), 0x00)
		dtc.severity.maintenance_only=True
		self.assertEqual(dtc.severity.get_byte_as_int(), 0x20)
		dtc.severity.check_at_next_exit = True
		self.assertEqual(dtc.severity.get_byte_as_int(), 0x60)
		dtc.severity.check_immediately = True
		self.assertEqual(dtc.severity.get_byte_as_int(), 0xE0)
		
		dtc.severity.set_byte(0x20)
		self.assertEqual(dtc.severity.maintenance_only, True)
		self.assertEqual(dtc.severity.check_at_next_exit, False)
		self.assertEqual(dtc.severity.check_immediately, False)
		
		dtc.severity.set_byte(0x60)
		self.assertEqual(dtc.severity.maintenance_only, True)
		self.assertEqual(dtc.severity.check_at_next_exit, True)
		self.assertEqual(dtc.severity.check_immediately, False)
		
		dtc.severity.set_byte(0xE0)
		self.assertEqual(dtc.severity.maintenance_only, True)
		self.assertEqual(dtc.severity.check_at_next_exit, True)
		self.assertEqual(dtc.severity.check_immediately, True)
		
class TestCodec(UdsTest):
	def test_DIDCodec_bad_values(self):
		with self.assertRaises(NotImplementedError):
			codec = DidCodec();
			codec.encode("asd")

		with self.assertRaises(NotImplementedError):
			codec = DidCodec();
			codec.decode(b"asd")

		with self.assertRaises(ValueError):
			DidCodec.from_config("")

	def test_ascii_codec(self):
		codec = AsciiCodec(10)
		self.assertEqual(codec.encode("abcdefghij"), b'abcdefghij');
		self.assertEqual(codec.decode(b"abcdefghij"), 'abcdefghij');

		with self.assertRaises(ValueError):
			codec.encode("abc")

		with self.assertRaises(ValueError):
			codec.encode("abcdefghijklmnop")

		with self.assertRaises(ValueError):
			AsciiCodec()



from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *
import struct

class WriteDataByIdentifier(BaseService):
	_sid = 0x2E
	_use_subfunction = False

	supported_negative_response = [	 Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.ConditionsNotCorrect,
							Response.Code.RequestOutOfRange,
							Response.Code.SecurityAccessDenied,
							Response.Code.GeneralProgrammingFailure
							]

	@classmethod
	def make_request(cls, did, value, didconfig):
		"""
		Generates a request for WriteDataByIdentifier

		:param did: The data identifier to write
		:type did: int

		:param value: Value given to the :ref:`DidCodec <DidCodec>`.encode method
		:type value: object

		:param didconfig: Definition of DID codecs. Dictionary mapping a DID (int) to a valid :ref:`DidCodec <DidCodec>` class or pack/unpack string 
		:type didconfig: dict[int] = :ref:`DidCodec <DidCodec>`

		:raises ValueError: If parameters are out of range, missing or wrong type
		:raises ConfigError: If ``didlist`` contains a DID not defined in ``didconfig``
		"""	

		from udsoncan import Request, DidCodec
		ServiceHelper.validate_int(did, min=0, max=0xFFFF, name='Data Identifier')
		req = Request(cls)
		didconfig = ServiceHelper.check_did_config(did, didconfig=didconfig)	# Make sure all DIDs are correctly defined in client config
		req.data = struct.pack('>H', did)	# encode DID number
		codec = DidCodec.from_config(didconfig[did])
		req.data += codec.encode(value)

		return req

	@classmethod
	def interpret_response(cls, response):
		"""
		Populates the response ``service_data`` property with an instance of :class:`WriteDataByIdentifier.ResponseData<udsoncan.services.WriteDataByIdentifier.ResponseData>`

		:param response: The received response to interpret
		:type response: :ref:`Response<Response>`
		
		:raises InvalidResponseException: If length of ``response.data`` is too short
		"""			
		if len(response.data) < 2:
			raise InvalidResponseException(response, "Response must be at least 2 bytes long")

		response.service_data = cls.ResponseData()
		response.service_data.did_echo = struct.unpack(">H", response.data[0:2])[0]

		
	class ResponseData(BaseResponseData):
		"""
		.. data:: did_echo

			The DID echoed back by the server
		"""			
		def __init__(self):
			super().__init__(WriteDataByIdentifier)

			self.did_echo = None


from . import *
from udsoncan.Response import Response
from udsoncan.exceptions import *

class SecuredDataTransmission(BaseService):
	_sid = 0x84

	class Code:
		GeneralSecurityViolation 			= Response.Code.GeneralSecurityViolation			- 0x38
		SecuredModeRequested 				= Response.Code.SecuredModeRequested				- 0x38
		InsufficientProtection 				= Response.Code.InsufficientProtection				- 0x38
		TerminationWithSignatureRequested 	= Response.Code.TerminationWithSignatureRequested	- 0x38
		AccessDenied 						= Response.Code.AccessDenied						- 0x38
		VersionNotSupported 				= Response.Code.VersionNotSupported					- 0x38
		SecuredLinkNotSupported 			= Response.Code.SecuredLinkNotSupported				- 0x38
		CertificateNotAvailable 			= Response.Code.CertificateNotAvailable				- 0x38
		AuditTrailInformationNotAvailable 	= Response.Code.AuditTrailInformationNotAvailable	- 0x38

	supported_negative_response = [	Response.Code.SubFunctionNotSupported, 
							Response.Code.IncorrectMessageLegthOrInvalidFormat,
							Response.Code.GeneralSecurityViolation,
							Response.Code.SecuredModeRequested,
							Response.Code.InsufficientProtection,
							Response.Code.TerminationWithSignatureRequested,
							Response.Code.AccessDenied,
							Response.Code.VersionNotSupported,
							Response.Code.SecuredLinkNotSupported,
							Response.Code.CertificateNotAvailable,
							Response.Code.AuditTrailInformationNotAvailable
							]

	@classmethod
	def make_request(cls):
		raise NotImplementedError('Service is not implemented')

	@classmethod
	def interpret_response(cls, response):
		raise NotImplementedError('Service is not implemented')

	class ResponseData(BaseResponseData):	
		def __init__(self):
			super().__init__(SecuredDataTransmission)
